# SPDX-License-Identifier: GPL-2.0
import os
import unittest

from packaging import version

from tests.integration.test_utils import PodmanAwareRunSubprocessMixin
from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import get_podman_version
from tests.integration.test_utils import is_systemd_available
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path(suffix: str = "") -> str:
    return os.path.join(os.path.join(test_path(), "deps"), f"docker-compose{suffix}.yaml")


class TestComposeBaseDeps(unittest.TestCase, RunSubprocessMixin):
    @unittest.skipIf(get_podman_version() >= version.parse("5.0.0"), "Breaks as of podman-5.4.2.")
    def test_deps(self) -> None:
        try:
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "run",
                "--rm",
                "sleep",
                "/bin/sh",
                "-c",
                "wget -O - http://web:8000/hosts",
            ])
            self.assertIn(b"HTTP request sent, awaiting response... 200 OK", output)
            self.assertIn(b"deps_web_1", output)
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
            ])

    def test_run_nodeps(self) -> None:
        try:
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "run",
                "--rm",
                "--no-deps",
                "sleep",
                "/bin/sh",
                "-c",
                "wget -O - http://web:8000/hosts || echo Failed to connect",
            ])
            self.assertNotIn(b"HTTP request sent, awaiting response... 200 OK", output)
            self.assertNotIn(b"deps_web_1", output)
            self.assertIn(b"Failed to connect", output)
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
            ])

    def test_up_nodeps(self) -> None:
        try:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "up",
                "--no-deps",
                "--detach",
                "sleep",
            ])
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "ps",
            ])
            self.assertNotIn(b"deps_web_1", output)
            self.assertIn(b"deps_sleep_1", output)
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
            ])

    @unittest.skipIf(get_podman_version() >= version.parse("5.0.0"), "Breaks as of podman-5.4.2.")
    def test_podman_compose_run(self) -> None:
        """
        This will test depends_on as well
        """
        run_cmd = [
            "coverage",
            "run",
            podman_compose_path(),
            "-f",
            os.path.join(test_path(), "deps", "docker-compose.yaml"),
            "run",
            "--rm",
            "sleep",
            "/bin/sh",
            "-c",
            "wget -q -O - http://web:8000/hosts",
        ]

        out, _ = self.run_subprocess_assert_returncode(run_cmd)
        self.assertIn(b"127.0.0.1\tlocalhost", out)

        # Run it again to make sure we can run it twice. I saw an issue where a second run, with
        # the container left up, would fail
        run_cmd = [
            "coverage",
            "run",
            podman_compose_path(),
            "-f",
            os.path.join(test_path(), "deps", "docker-compose.yaml"),
            "run",
            "--rm",
            "sleep",
            "/bin/sh",
            "-c",
            "wget -q -O - http://web:8000/hosts",
        ]

        out, _ = self.run_subprocess_assert_returncode(run_cmd)
        self.assertIn(b"127.0.0.1\tlocalhost", out)

        # This leaves a container running. Not sure it's intended, but it matches docker-compose
        down_cmd = [
            "coverage",
            "run",
            podman_compose_path(),
            "-f",
            os.path.join(test_path(), "deps", "docker-compose.yaml"),
            "down",
        ]

        self.run_subprocess_assert_returncode(down_cmd)


class TestComposeConditionalDeps(unittest.TestCase, RunSubprocessMixin):
    @unittest.skipIf(get_podman_version() >= version.parse("5.0.0"), "Breaks as of podman-5.4.2.")
    def test_deps_succeeds(self) -> None:
        suffix = "-conditional-succeeds"
        try:
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(suffix),
                "run",
                "--rm",
                "sleep",
                "/bin/sh",
                "-c",
                "wget -O - http://web:8000/hosts",
            ])
            self.assertIn(b"HTTP request sent, awaiting response... 200 OK", output)
            self.assertIn(b"deps_web_1", output)
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(suffix),
                "down",
            ])

    def test_deps_fails(self) -> None:
        suffix = "-conditional-fails"
        try:
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(suffix),
                "ps",
            ])
            self.assertNotIn(b"HTTP request sent, awaiting response... 200 OK", output)
            self.assertNotIn(b"deps_web_1", output)
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(suffix),
                "down",
            ])


class TestComposeConditionalDepsHealthy(unittest.TestCase, PodmanAwareRunSubprocessMixin):
    def setUp(self) -> None:
        self.podman_version = self.retrieve_podman_version()

    @unittest.skipIf(
        get_podman_version() > version.parse("4.4.0"), "Breaks as of podman-4.9.5 and podman-5.4.2."
    )
    def test_up_deps_healthy(self) -> None:
        suffix = "-conditional-healthy"
        try:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(suffix),
                "up",
                "sleep",
                "--detach",
            ])

            # Since the command `podman wait --condition=healthy` is invalid prior to 4.6.0,
            # we only validate healthy status for podman 4.6.0+, which won't be tested in the
            # CI pipeline of the podman-compose project where podman 4.3.1 is employed.
            podman_ver_major, podman_ver_minor, podman_ver_patch = self.podman_version
            if podman_ver_major >= 4 and podman_ver_minor >= 6 and podman_ver_patch >= 0:
                self.run_subprocess_assert_returncode([
                    "podman",
                    "wait",
                    "--condition=running",
                    "deps_web_1",
                    "deps_sleep_1",
                ])

            # check both web and sleep are running
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "ps",
                "--format",
                "{{.ID}}\t{{.Names}}\t{{.Status}}\t{{.StartedAt}}",
            ])

            # extract container id of web
            decoded_out = output.decode('utf-8')
            lines = decoded_out.split("\n")

            web_lines = [line for line in lines if "web" in line]
            self.assertTrue(web_lines)
            self.assertEqual(1, len(web_lines))
            web_cnt_id, web_cnt_name, web_cnt_status, web_cnt_started = web_lines[0].split("\t")
            self.assertNotEqual("", web_cnt_id)
            self.assertEqual("deps_web_1", web_cnt_name)

            sleep_lines = [line for line in lines if "sleep" in line]
            self.assertTrue(sleep_lines)
            self.assertEqual(1, len(sleep_lines))
            sleep_cnt_id, sleep_cnt_name, _, sleep_cnt_started = sleep_lines[0].split("\t")
            self.assertNotEqual("", sleep_cnt_id)
            self.assertEqual("deps_sleep_1", sleep_cnt_name)

            # When test case is executed inside container like github actions, the absence of
            # systemd prevents health check from working properly, resulting in failure to
            # transit to healthy state. As a result, we only assert the `healthy` state where
            # systemd is functioning.
            if (
                is_systemd_available()
                and podman_ver_major >= 4
                and podman_ver_minor >= 6
                and podman_ver_patch >= 0
            ):
                self.assertIn("healthy", web_cnt_status)
            self.assertGreaterEqual(int(sleep_cnt_started), int(web_cnt_started))

        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(suffix),
                "down",
            ])



# SPDX-License-Identifier: GPL-2.0

# pylint: disable=redefined-outer-name
import os
import unittest

from packaging import version

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import get_podman_version
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


class TestPodmanComposeNetworkScopedAliases(RunSubprocessMixin, unittest.TestCase):
    @staticmethod
    def compose_file() -> str:
        """Returns the path to the compose file used for this test module"""
        return os.path.join(test_path(), "network_scoped_aliases", "docker-compose.yaml")

    @unittest.skipIf(get_podman_version() >= version.parse("5.0.0"), "Breaks as of podman-5.4.2.")
    def test_network_scoped_aliases(self) -> None:
        try:
            self.up()
            self.verify()
        finally:
            self.down()

    def up(self) -> None:
        up_cmd = [
            "coverage",
            "run",
            podman_compose_path(),
            "-f",
            self.compose_file(),
            "up",
            "-d",
            "--force-recreate",
        ]

        self.run_subprocess_assert_returncode(up_cmd)

    def down(self) -> None:
        down_cmd = [
            "coverage",
            "run",
            podman_compose_path(),
            "-f",
            self.compose_file(),
            "kill",
            "-a",
        ]
        self.run_subprocess(down_cmd)

    def verify(self) -> None:
        expected_results = [
            ("utils-net0", "web1", ["172.19.3.11"]),
            ("utils-net0", "secure-web", ["172.19.3.11"]),
            ("utils-net0", "insecure-web", []),
            ("utils-net1", "web1", ["172.19.4.11"]),
            ("utils-net1", "secure-web", []),
            ("utils-net1", "insecure-web", ["172.19.4.11"]),
        ]

        for utils, service, expected_result in expected_results:
            cmd = [
                podman_compose_path(),
                "-f",
                self.compose_file(),
                "exec",
                utils,
                "nslookup",
                service,
            ]
            out, _, _ = self.run_subprocess(cmd)
            addresses = self.parse_dnslookup(out.decode())
            self.assertEqual(addresses, expected_result)

    def parse_dnslookup(self, output: str) -> list[str]:
        lines = output.splitlines()
        addresses = []
        for line in lines:
            if line.startswith("Address"):
                addr = line.split(":", 1)[1].strip()
                if ":" not in addr:
                    addresses.append(addr)

        return list(sorted(set(addresses)))



# SPDX-License-Identifier: GPL-2.0

import os
import unittest

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path() -> str:
    return os.path.join(os.path.join(test_path(), "multicompose"), "docker-compose.yml")


class TestComposeMulticompose(unittest.TestCase, RunSubprocessMixin):
    def test_multicompose(self) -> None:
        try:
            self.run_subprocess_assert_returncode(
                [
                    podman_compose_path(),
                    "-f",
                    os.path.join(
                        os.path.join(test_path(), "multicompose"), "d1/docker-compose.yml"
                    ),
                    "-f",
                    os.path.join(
                        os.path.join(test_path(), "multicompose"), "d2/docker-compose.yml"
                    ),
                    "up",
                    "-d",
                ],
            )
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                os.path.join(os.path.join(test_path(), "multicompose"), "d1/docker-compose.yml"),
                "-f",
                os.path.join(os.path.join(test_path(), "multicompose"), "d2/docker-compose.yml"),
                "ps",
            ])
            self.assertIn(b"d1_web1_1", output)
            self.assertIn(b"d1_web2_1", output)

            output, _ = self.run_subprocess_assert_returncode([
                "podman",
                "exec",
                "-ti",
                "d1_web1_1",
                "sh",
                "-c",
                "set",
            ])
            # checks if `env_file` was appended, not replaced
            # (which means that we normalize to array before merge)
            self.assertIn(b"var12='d1/12.env'", output)

            output, _ = self.run_subprocess_assert_returncode([
                "podman",
                "exec",
                "-ti",
                "d1_web2_1",
                "sh",
                "-c",
                "set",
            ])
            # checks if paths inside `d2/docker-compose.yml` directory are relative to `d1`
            self.assertIn(b"var2='d1/2.env'", output)

            output, _ = self.run_subprocess_assert_returncode([
                "podman",
                "exec",
                "-ti",
                "d1_web1_1",
                "sh",
                "-c",
                "cat /var/www/html/index.txt",
            ])
            self.assertIn(b"var1=d1/1.env", output)

            # check if project base directory and project name is d1
            output, _ = self.run_subprocess_assert_returncode([
                "podman",
                "exec",
                "-ti",
                "d1_web2_1",
                "sh",
                "-c",
                "cat /var/www/html/index.txt",
            ])
            self.assertIn(b"var2=d1/2.env", output)
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                os.path.join(os.path.join(test_path(), "multicompose"), "d1/docker-compose.yml"),
                "-f",
                os.path.join(os.path.join(test_path(), "multicompose"), "d2/docker-compose.yml"),
                "down",
                "-t",
                "0",
            ])



import unittest
from typing import Any

from podman_compose import get_network_create_args


class TestGetNetworkCreateArgs(unittest.TestCase):
    def get_minimal_net_desc(self) -> dict[str, Any]:
        return {
            "labels": [],
            "internal": False,
            "driver": None,
            "driver_opts": {},
            "ipam": {"config": []},
            "enable_ipv6": False,
        }

    def test_minimal(self) -> None:
        net_desc = self.get_minimal_net_desc()
        proj_name = "test_project"
        net_name = "test_network"
        expected_args = [
            "create",
            "--label",
            f"io.podman.compose.project={proj_name}",
            "--label",
            f"com.docker.compose.project={proj_name}",
            net_name,
        ]
        args = get_network_create_args(net_desc, proj_name, net_name)
        self.assertEqual(args, expected_args)

    def test_ipv6(self) -> None:
        net_desc = self.get_minimal_net_desc()
        net_desc["enable_ipv6"] = True
        proj_name = "test_project"
        net_name = "test_network"
        expected_args = [
            "create",
            "--label",
            f"io.podman.compose.project={proj_name}",
            "--label",
            f"com.docker.compose.project={proj_name}",
            "--ipv6",
            net_name,
        ]
        args = get_network_create_args(net_desc, proj_name, net_name)
        self.assertEqual(args, expected_args)

    def test_bridge(self) -> None:
        net_desc = self.get_minimal_net_desc()
        net_desc["driver"] = "bridge"
        net_desc["driver_opts"] = {"opt1": "value1", "opt2": "value2"}
        proj_name = "test_project"
        net_name = "test_network"
        expected_args = [
            "create",
            "--label",
            f"io.podman.compose.project={proj_name}",
            "--label",
            f"com.docker.compose.project={proj_name}",
            "--driver",
            "bridge",
            "--opt",
            "opt1=value1",
            "--opt",
            "opt2=value2",
            net_name,
        ]
        args = get_network_create_args(net_desc, proj_name, net_name)
        self.assertEqual(args, expected_args)

    def test_ipam_driver_default(self) -> None:
        net_desc = self.get_minimal_net_desc()
        net_desc["ipam"] = {
            "driver": "default",
            "config": [
                {
                    "subnet": "192.168.0.0/24",
                    "ip_range": "192.168.0.2/24",
                    "gateway": "192.168.0.1",
                }
            ],
        }
        proj_name = "test_project"
        net_name = "test_network"
        expected_args = [
            "create",
            "--label",
            f"io.podman.compose.project={proj_name}",
            "--label",
            f"com.docker.compose.project={proj_name}",
            "--subnet",
            "192.168.0.0/24",
            "--ip-range",
            "192.168.0.2/24",
            "--gateway",
            "192.168.0.1",
            net_name,
        ]
        args = get_network_create_args(net_desc, proj_name, net_name)
        self.assertEqual(args, expected_args)

    def test_ipam_driver(self) -> None:
        net_desc = self.get_minimal_net_desc()
        net_desc["ipam"] = {
            "driver": "someipamdriver",
            "config": [
                {
                    "subnet": "192.168.0.0/24",
                    "ip_range": "192.168.0.2/24",
                    "gateway": "192.168.0.1",
                }
            ],
        }
        proj_name = "test_project"
        net_name = "test_network"
        expected_args = [
            "create",
            "--label",
            f"io.podman.compose.project={proj_name}",
            "--label",
            f"com.docker.compose.project={proj_name}",
            "--ipam-driver",
            "someipamdriver",
            "--subnet",
            "192.168.0.0/24",
            "--ip-range",
            "192.168.0.2/24",
            "--gateway",
            "192.168.0.1",
            net_name,
        ]
        args = get_network_create_args(net_desc, proj_name, net_name)
        self.assertEqual(args, expected_args)

    def test_complete(self) -> None:
        net_desc = {
            "labels": ["label1", "label2"],
            "internal": True,
            "driver": "bridge",
            "driver_opts": {"opt1": "value1", "opt2": "value2"},
            "ipam": {
                "driver": "someipamdriver",
                "config": [
                    {
                        "subnet": "192.168.0.0/24",
                        "ip_range": "192.168.0.2/24",
                        "gateway": "192.168.0.1",
                    }
                ],
            },
            "enable_ipv6": True,
        }
        proj_name = "test_project"
        net_name = "test_network"
        expected_args = [
            "create",
            "--label",
            f"io.podman.compose.project={proj_name}",
            "--label",
            f"com.docker.compose.project={proj_name}",
            "--label",
            "label1",
            "--label",
            "label2",
            "--internal",
            "--driver",
            "bridge",
            "--opt",
            "opt1=value1",
            "--opt",
            "opt2=value2",
            "--ipam-driver",
            "someipamdriver",
            "--ipv6",
            "--subnet",
            "192.168.0.0/24",
            "--ip-range",
            "192.168.0.2/24",
            "--gateway",
            "192.168.0.1",
            net_name,
        ]
        args = get_network_create_args(net_desc, proj_name, net_name)
        self.assertEqual(args, expected_args)

    def test_disable_dns(self) -> None:
        net_desc = self.get_minimal_net_desc()
        net_desc["x-podman.disable_dns"] = True
        proj_name = "test_project"
        net_name = "test_network"
        expected_args = [
            "create",
            "--label",
            f"io.podman.compose.project={proj_name}",
            "--label",
            f"com.docker.compose.project={proj_name}",
            "--disable-dns",
            net_name,
        ]
        args = get_network_create_args(net_desc, proj_name, net_name)
        self.assertEqual(args, expected_args)

    def test_dns_string(self) -> None:
        net_desc = self.get_minimal_net_desc()
        net_desc["x-podman.dns"] = "192.168.1.2"
        proj_name = "test_project"
        net_name = "test_network"
        expected_args = [
            "create",
            "--label",
            f"io.podman.compose.project={proj_name}",
            "--label",
            f"com.docker.compose.project={proj_name}",
            "--dns",
            "192.168.1.2",
            net_name,
        ]
        args = get_network_create_args(net_desc, proj_name, net_name)
        self.assertEqual(args, expected_args)

    def test_dns_list(self) -> None:
        net_desc = self.get_minimal_net_desc()
        net_desc["x-podman.dns"] = ["192.168.1.2", "192.168.1.3"]
        proj_name = "test_project"
        net_name = "test_network"
        expected_args = [
            "create",
            "--label",
            f"io.podman.compose.project={proj_name}",
            "--label",
            f"com.docker.compose.project={proj_name}",
            "--dns",
            "192.168.1.2,192.168.1.3",
            net_name,
        ]
        args = get_network_create_args(net_desc, proj_name, net_name)
        self.assertEqual(args, expected_args)

    def test_routes_string(self) -> None:
        net_desc = self.get_minimal_net_desc()
        net_desc["x-podman.routes"] = "192.168.1.0/24"
        proj_name = "test_project"
        net_name = "test_network"
        expected_args = [
            "create",
            "--label",
            f"io.podman.compose.project={proj_name}",
            "--label",
            f"com.docker.compose.project={proj_name}",
            "--route",
            "192.168.1.0/24",
            net_name,
        ]
        args = get_network_create_args(net_desc, proj_name, net_name)
        self.assertEqual(args, expected_args)

    def test_routes_list(self) -> None:
        net_desc = self.get_minimal_net_desc()
        net_desc["x-podman.routes"] = ["192.168.1.0/24", "192.168.2.0/24"]
        proj_name = "test_project"
        net_name = "test_network"
        expected_args = [
            "create",
            "--label",
            f"io.podman.compose.project={proj_name}",
            "--label",
            f"com.docker.compose.project={proj_name}",
            "--route",
            "192.168.1.0/24",
            "--route",
            "192.168.2.0/24",
            net_name,
        ]
        args = get_network_create_args(net_desc, proj_name, net_name)
        self.assertEqual(args, expected_args)



# SPDX-License-Identifier: GPL-2.0

"""
test_podman_compose_vol.py

Tests the podman compose up and down commands used to create and remove services with volumes.
"""

# pylint: disable=redefined-outer-name
import os
import unittest

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


class TestPodmanCompose(unittest.TestCase, RunSubprocessMixin):
    def test_down_with_vols(self) -> None:
        up_cmd = [
            "coverage",
            "run",
            podman_compose_path(),
            "-f",
            os.path.join(test_path(), "vol", "docker-compose.yaml"),
            "up",
            "-d",
        ]

        down_cmd = [
            "coverage",
            "run",
            podman_compose_path(),
            "-f",
            os.path.join(test_path(), "vol", "docker-compose.yaml"),
            "down",
            "--volumes",
        ]

        try:
            self.run_subprocess_assert_returncode(["podman", "volume", "create", "my-app-data"])
            self.run_subprocess_assert_returncode([
                "podman",
                "volume",
                "create",
                "actual-name-of-volume",
            ])

            self.run_subprocess_assert_returncode(up_cmd)
            self.run_subprocess(["podman", "inspect", "volume", ""])

        finally:
            out, _, return_code = self.run_subprocess(down_cmd)
            self.run_subprocess(["podman", "volume", "rm", "my-app-data"])
            self.run_subprocess(["podman", "volume", "rm", "actual-name-of-volume"])
            self.assertEqual(return_code, 0)



# SPDX-License-Identifier: GPL-2.0

"""
test_podman_compose_config.py

Tests the podman-compose config command which is used to return defined compose services.
"""

# pylint: disable=redefined-outer-name
import os
import unittest

from parameterized import parameterized

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def profile_compose_file() -> str:
    """ "Returns the path to the `profile` compose file used for this test module"""
    return os.path.join(test_path(), "profile", "docker-compose.yml")


class TestComposeConfig(unittest.TestCase, RunSubprocessMixin):
    def test_config_no_profiles(self) -> None:
        """
        Tests podman-compose config command without profile enablement.
        """
        config_cmd = [
            "coverage",
            "run",
            podman_compose_path(),
            "-f",
            profile_compose_file(),
            "config",
        ]

        out, _ = self.run_subprocess_assert_returncode(config_cmd)

        string_output = out.decode("utf-8")
        self.assertIn("default-service", string_output)
        self.assertNotIn("service-1", string_output)
        self.assertNotIn("service-2", string_output)

    @parameterized.expand(
        [
            (
                ["--profile", "profile-1", "config"],
                {"default-service": True, "service-1": True, "service-2": False},
            ),
            (
                ["--profile", "profile-2", "config"],
                {"default-service": True, "service-1": False, "service-2": True},
            ),
            (
                ["--profile", "profile-1", "--profile", "profile-2", "config"],
                {"default-service": True, "service-1": True, "service-2": True},
            ),
        ],
    )
    def test_config_profiles(self, profiles: list, expected_services: dict) -> None:
        """
        Tests podman-compose
        :param profiles: The enabled profiles for the parameterized test.
        :param expected_services: Dictionary used to model the expected "enabled" services in the
            profile. Key = service name, Value = True if the service is enabled, otherwise False.
        """
        config_cmd = ["coverage", "run", podman_compose_path(), "-f", profile_compose_file()]
        config_cmd.extend(profiles)

        out, _ = self.run_subprocess_assert_returncode(config_cmd)

        actual_output = out.decode("utf-8")

        self.assertEqual(len(expected_services), 3)

        actual_services = {}
        for service, _ in expected_services.items():
            actual_services[service] = service in actual_output

        self.assertEqual(expected_services, actual_services)



# SPDX-License-Identifier: GPL-2.0
# pylint: disable=protected-access

import unittest
from typing import Any

from parameterized import parameterized

from podman_compose import rec_subs


class TestRecSubs(unittest.TestCase):
    substitutions = [
        # dict with environment variables
        (
            "service's environment is low priority",
            {"environment": {"v1": "low priority", "actual-v1": "$v1"}},
            {"environment": {"v1": "low priority", "actual-v1": "high priority"}},
        ),
        (
            "service's environment can be used in other values",
            {"environment": {"v100": "v1.0.0", "image": "abc:$v100"}},
            {"environment": {"v100": "v1.0.0", "image": "abc:v1.0.0"}},
        ),
        (
            "Non-variable should not be substituted",
            {"environment": {"non_var": "$$v1", "vx": "$non_var"}, "image": "abc:$non_var"},
            {"environment": {"non_var": "$v1", "vx": "$v1"}, "image": "abc:$v1"},
        ),
        (
            "service environment with unpopulated ${VARIABLE:-default} format",
            {"environment": {"v100": "${v100:-low priority}", "actual-v100": "$v100"}},
            {"environment": {"v100": "low priority", "actual-v100": "low priority"}},
        ),
        (
            "service environment with populated ${VARIABLE:-default} format",
            {"environment": {"v1": "${v1:-low priority}", "actual-v1": "$v1"}},
            {"environment": {"v1": "high priority", "actual-v1": "high priority"}},
        ),
        # list
        (
            "Values in list are substituted",
            ["$v1", "low priority"],
            ["high priority", "low priority"],
        ),
        # str
        (
            "Value with ${VARIABLE} format",
            "${v1}",
            "high priority",
        ),
        (
            "Value with ${VARIABLE:-default} format",
            ["${v1:-default}", "${empty:-default}", "${not_exits:-default}"],
            ["high priority", "default", "default"],
        ),
        (
            "Value with ${VARIABLE-default} format",
            ["${v1-default}", "${empty-default}", "${not_exits-default}"],
            ["high priority", "", "default"],
        ),
        (
            "Value $$ means $",
            "$$v1",
            "$v1",
        ),
    ]

    @parameterized.expand(substitutions)
    def test_rec_subs(self, desc: str, input: Any, expected: Any) -> None:
        sub_dict = {"v1": "high priority", "empty": ""}
        result = rec_subs(input, sub_dict)
        self.assertEqual(result, expected, msg=desc)

    def test_env_var_substitution_in_dictionary_keys(self) -> None:
        sub_dict = {"NAME": "TEST1", "NAME2": "TEST2"}
        input = {
            'services': {
                'test': {
                    'image': 'busybox',
                    'labels': {
                        '$NAME and ${NAME2}': '${NAME2} and $NAME',
                        'test1.${NAME}': 'test1',
                        '$NAME': '${NAME2}',
                        '${NAME}.test2': 'Host(`${NAME2}`)',
                    },
                }
            }
        }
        result = rec_subs(input, sub_dict)
        expected = {
            'services': {
                'test': {
                    'image': 'busybox',
                    'labels': {
                        'TEST1 and TEST2': 'TEST2 and TEST1',
                        'test1.TEST1': 'test1',
                        'TEST1': 'TEST2',
                        'TEST1.test2': 'Host(`TEST2`)',
                    },
                }
            }
        }
        self.assertEqual(result, expected)



# SPDX-License-Identifier: GPL-2.0

import argparse
import unittest

from podman_compose import PodmanCompose
from podman_compose import compose_run_update_container_from_args


class TestComposeRunUpdateContainerFromArgs(unittest.TestCase):
    def test_minimal(self) -> None:
        cnt = get_minimal_container()
        compose = get_minimal_compose()
        args = get_minimal_args()

        compose_run_update_container_from_args(compose, cnt, args)

        expected_cnt = {"name": "default_name", "tty": True}
        self.assertEqual(cnt, expected_cnt)

    def test_additional_env_value_equals(self) -> None:
        cnt = get_minimal_container()
        compose = get_minimal_compose()
        args = get_minimal_args()
        args.env = ["key=valuepart1=valuepart2"]

        compose_run_update_container_from_args(compose, cnt, args)

        expected_cnt = {
            "environment": {
                "key": "valuepart1=valuepart2",
            },
            "name": "default_name",
            "tty": True,
        }
        self.assertEqual(cnt, expected_cnt)

    def test_publish_ports(self) -> None:
        cnt = get_minimal_container()
        compose = get_minimal_compose()
        args = get_minimal_args()
        args.publish = ["1111", "2222:2222"]

        compose_run_update_container_from_args(compose, cnt, args)

        expected_cnt = {
            "name": "default_name",
            "ports": ["1111", "2222:2222"],
            "tty": True,
        }
        self.assertEqual(cnt, expected_cnt)


def get_minimal_container() -> dict:
    return {}


def get_minimal_compose() -> PodmanCompose:
    compose = PodmanCompose()
    compose.project_name = "test_project"
    return compose


def get_minimal_args() -> argparse.Namespace:
    return argparse.Namespace(
        T=None,
        cnt_command=None,
        entrypoint=None,
        env=None,
        name="default_name",
        rm=None,
        service="test_service",
        publish=None,
        service_ports=None,
        user=None,
        volume=None,
        workdir=None,
    )



# SPDX-License-Identifier: GPL-2.0

import os
import unittest
import uuid

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path() -> str:
    base_path = os.path.join(test_path(), "longlog")
    return os.path.join(base_path, "docker-compose-multibyte.yml")


class TestLongLog(unittest.TestCase, RunSubprocessMixin):
    def test_print_until_end_of_multibyte_output(self) -> None:
        # adding uuid because hangup containers can interfere next test runs
        project_id = f"longlog_{uuid.uuid4().hex[:8]}"
        try:
            out, err = self.run_subprocess_assert_returncode(
                [
                    podman_compose_path(),
                    "-p",
                    project_id,
                    "-f",
                    compose_yaml_path(),
                    "up",
                ],
                0,
                timeout=6.0,
            )
            self.assertEqual(b"", err)
            outstring = out.decode("utf-8")
            self.assertTrue(
                outstring.endswith(" end\n"), f"ending with {outstring[-20:].encode('utf-8')!r}"
            )
        finally:
            self.run_subprocess([
                podman_compose_path(),
                "-p",
                project_id,
                "-f",
                compose_yaml_path(),
                "down",
            ])



# SPDX-License-Identifier: GPL-2.0

import json
import os
import unittest

from packaging import version

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import get_podman_version
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path() -> str:
    return os.path.join(
        test_path(),
        "merge/reset_and_override_tags/override_tag_service/docker-compose.yaml",
    )


class TestComposeOverrideTagService(unittest.TestCase, RunSubprocessMixin):
    # test if whole service from docker-compose.yaml file is overridden in another file
    @unittest.skipIf(get_podman_version() >= version.parse("5.0.0"), "Breaks as of podman-5.4.2.")
    def test_override_tag_service(self) -> None:
        override_file = os.path.join(
            test_path(),
            "merge/reset_and_override_tags/override_tag_service/docker-compose.override_service.yaml",
        )
        try:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "-f",
                override_file,
                "up",
            ])

            # Whole app service was overridden in the docker-compose.override_tag_service.yaml file.
            # Command and port is overridden accordingly.
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "-f",
                override_file,
                "logs",
                "--no-log-prefix",
                "--no-color",
            ])
            self.assertEqual(output, b"One\n")

            output, _ = self.run_subprocess_assert_returncode([
                "podman",
                "inspect",
                "override_tag_service_app_1",
            ])
            container_info = json.loads(output.decode('utf-8'))[0]
            self.assertEqual(
                container_info['NetworkSettings']["Ports"],
                {"81/tcp": [{"HostIp": "", "HostPort": "8111"}]},
            )
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
            ])



# SPDX-License-Identifier: GPL-2.0

# pylint: disable=redefined-outer-name
import os
import unittest

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


class TestPodmanComposeNetworkInterfaceName(RunSubprocessMixin, unittest.TestCase):
    def compose_file(self) -> str:
        return os.path.join(test_path(), "network_interface_name", "docker-compose.yml")

    def up(self) -> None:
        up_cmd = [
            "coverage",
            "run",
            podman_compose_path(),
            "-f",
            self.compose_file(),
            "up",
            "-d",
            "--force-recreate",
        ]
        self.run_subprocess_assert_returncode(up_cmd)

    def down(self) -> None:
        down_cmd = [
            "coverage",
            "run",
            podman_compose_path(),
            "-f",
            self.compose_file(),
            "kill",
            "-a",
        ]
        self.run_subprocess(down_cmd)

    def test_interface_name(self) -> None:
        try:
            self.up()

            interfaces_cmd = [
                podman_compose_path(),
                "-f",
                self.compose_file(),
                "exec",
                "web",
                "ls",
                "/sys/class/net",
                "--color=never",
            ]
            out, _ = self.run_subprocess_assert_returncode(interfaces_cmd)
            self.assertEqual("customName0  lo\r\n", out.decode())
        finally:
            self.down()



# SPDX-License-Identifier: GPL-2.0

import os
import unittest

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def networks_no_services_compose() -> str:
    return compose_yaml_path("docker-compose-networks-no-services.yaml")


def empty_services_compose() -> str:
    return compose_yaml_path("docker-compose-empty-services.yaml")


def compose_yaml_path(filename: str) -> str:
    return os.path.join(os.path.join(test_path(), "no_services"), filename)


class TestComposeNoServices(unittest.TestCase, RunSubprocessMixin):
    # test if a network was created, but not the services
    def test_no_services(self) -> None:
        try:
            _, err = self.run_subprocess_assert_returncode(
                [
                    podman_compose_path(),
                    "-f",
                    networks_no_services_compose(),
                    "up",
                    "-d",
                ],
            )
            self.assertIn(b'WARNING:__main__:WARNING: unused networks: shared-network\n', err)

            container_id, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                networks_no_services_compose(),
                "ps",
                "--format",
                '{{.ID}}',
            ])
            self.assertEqual(container_id, b"")
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                networks_no_services_compose(),
                "down",
                "-t",
                "0",
            ])

    def test_empty_services_no_stacktrace(self) -> None:
        try:
            _, err = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                empty_services_compose(),
                "up",
                "-d",
            ])
            self.assertNotIn(b"\nTraceback (most recent call last):\n", err)
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                empty_services_compose(),
                "down",
                "-t",
                "0",
            ])



# SPDX-License-Identifier: GPL-2.0

import os
import unittest

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path():
    """Returns the path to the compose file used for this test module"""
    base_path = os.path.join(test_path(), "build_fail")
    return os.path.join(base_path, "docker-compose.yml")


class TestComposeBuildFail(unittest.TestCase, RunSubprocessMixin):
    def test_build_fail(self):
        output, error = self.run_subprocess_assert_returncode(
            [
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "build",
                "test",
            ],
            expected_returncode=127,
        )
        self.assertIn("RUN this_command_does_not_exist", str(output))
        self.assertIn("this_command_does_not_exist: not found", str(error))
        self.assertIn("while running runtime: exit status 127", str(error))

    def test_dockerfile_does_not_exist(self):
        out, error = self.run_subprocess_assert_returncode(
            [
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "build",
                "test_no_dockerfile",
            ],
            expected_returncode=1,
        )
        error = error.decode('utf-8')
        result = '\n'.join(error.splitlines()[-1:])

        expected_path = os.path.join(os.path.dirname(__file__), "context_no_file")
        expected = f'OSError: Dockerfile not found in {expected_path}'

        self.assertEqual(expected, result)

    def test_custom_dockerfile_does_not_exist(self):
        out, error = self.run_subprocess_assert_returncode(
            [
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "build",
                "test_no_custom_dockerfile",
            ],
            expected_returncode=1,
        )
        error = error.decode('utf-8')
        result = '\n'.join(error.splitlines()[-1:])

        expected_path = os.path.join(os.path.dirname(__file__), "context_no_file/Dockerfile-alt")
        expected = f'OSError: Dockerfile not found in {expected_path}'

        self.assertEqual(expected, result)



# SPDX-License-Identifier: GPL-2.0

import os
import shutil
import unittest

from parameterized import parameterized

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path(test_ref_folder: str) -> str:
    return os.path.join(test_path(), "vol", test_ref_folder, "docker-compose.yml")


class TestComposeVolLongSyntax(unittest.TestCase, RunSubprocessMixin):
    @parameterized.expand([
        (True, "create_host_path_default_true"),
        (True, "create_host_path_true"),
        (True, "create_host_path_false"),
        (False, "create_host_path_default_true"),
        (False, "create_host_path_true"),
    ])
    def test_source_host_dir(self, source_dir_exists: bool, service_name: str) -> None:
        project_dir = os.path.join(test_path(), "vol", "long_syntax")
        if source_dir_exists:
            # create host source directory for volume
            os.mkdir(os.path.join(project_dir, "test_dir"))
        else:
            # make sure there is no such directory
            self.assertFalse(os.path.isdir(os.path.join(project_dir, "test_dir")))

        try:
            self.run_subprocess_assert_returncode(
                [
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path("long_syntax"),
                    "up",
                    "-d",
                    f"{service_name}",
                ],
                0,
            )
            # command of the service creates a new directory on mounted directory 'test_dir' in
            # the container. Check if host directory now has the same directory. It represents a
            # successful mount.
            # If source host directory does not exist, it is created
            self.assertTrue(os.path.isdir(os.path.join(project_dir, "test_dir/new_dir")))

        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path("long_syntax"),
                "down",
                "-t",
                "0",
            ])
            shutil.rmtree(os.path.join(project_dir, "test_dir"))

    def test_no_host_source_dir_create_host_path_false(self) -> None:
        try:
            _, error = self.run_subprocess_assert_returncode(
                [
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path("long_syntax"),
                    "up",
                    "-d",
                    "create_host_path_false",
                ],
                1,
            )
            self.assertIn(
                b"invalid mount config for type 'bind': bind source path does not exist:", error
            )
        finally:
            self.run_subprocess([
                podman_compose_path(),
                "-f",
                compose_yaml_path("long_syntax"),
                "down",
                "-t",
                "0",
            ])



# SPDX-License-Identifier: GPL-2.0

import os
import unittest

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path() -> str:
    """ "Returns the path to the compose file used for this test module"""
    base_path = os.path.join(test_path(), "commands_fail_exit_code")
    return os.path.join(base_path, "docker-compose.yml")


class TestConfigCommand(unittest.TestCase, RunSubprocessMixin):
    def test_config_quiet(self) -> None:
        """
        Tests podman-compose config command with the --quiet flag.
        """
        config_cmd = [
            "coverage",
            "run",
            podman_compose_path(),
            "-f",
            compose_yaml_path(),
            "config",
            "--quiet",
        ]

        out, _ = self.run_subprocess_assert_returncode(config_cmd)
        self.assertEqual(out.decode("utf-8"), "")



# SPDX-License-Identifier: GPL-2.0

"""
test_podman_compose_ports.py

Tests the podman compose port command used to show the host port.
"""

import os
import unittest

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


class TestPodmanCompose(unittest.TestCase, RunSubprocessMixin):
    def test_up_with_ports(self) -> None:
        up_cmd = [
            "coverage",
            "run",
            podman_compose_path(),
            "-f",
            os.path.join(test_path(), "ports", "docker-compose.yml"),
            "up",
            "-d",
            "--force-recreate",
        ]

        down_cmd = [
            "coverage",
            "run",
            podman_compose_path(),
            "-f",
            os.path.join(test_path(), "ports", "docker-compose.yml"),
            "down",
        ]

        port_cmd = [
            podman_compose_path(),
            "-f",
            os.path.join(test_path(), "ports", "docker-compose.yml"),
            "port",
        ]

        udp_arg = ["--protocol", "udp"]

        tcp_arg = ["--protocol", "tcp"]

        try:
            self.run_subprocess_assert_returncode(up_cmd)

            port = self.run_subprocess_assert_returncode(port_cmd + ["web1", "8000"])
            self.assertEqual(port[0].decode().strip(), "8000")

            port = self.run_subprocess_assert_returncode(port_cmd + ["web1", "8001"])
            self.assertNotEqual(port[0].decode().strip(), "8001")

            port = self.run_subprocess_assert_returncode(port_cmd + ["web2", "8002"])
            self.assertEqual(port[0].decode().strip(), "8002")

            port = self.run_subprocess_assert_returncode(port_cmd + udp_arg + ["web2", "8003"])
            self.assertEqual(port[0].decode().strip(), "8003")

            port = self.run_subprocess_assert_returncode(port_cmd + ["web2", "8004"])
            self.assertEqual(port[0].decode().strip(), "8004")

            port = self.run_subprocess_assert_returncode(port_cmd + tcp_arg + ["web2", "8005"])
            self.assertEqual(port[0].decode().strip(), "8005")

            port = self.run_subprocess_assert_returncode(port_cmd + udp_arg + ["web2", "8006"])
            self.assertNotEqual(port[0].decode().strip(), "8006")

            port = self.run_subprocess_assert_returncode(port_cmd + ["web2", "8007"])
            self.assertNotEqual(port[0].decode().strip(), "8007")

        finally:
            self.run_subprocess_assert_returncode(down_cmd)



# SPDX-License-Identifier: GPL-2.0

"""
test_podman_compose_up_down.py

Tests the podman compose up and down commands used to create and remove services.
"""

# pylint: disable=redefined-outer-name
import os
import unittest
from typing import List

from parameterized import parameterized

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def profile_compose_file() -> str:
    """ "Returns the path to the `profile` compose file used for this test module"""
    return os.path.join(test_path(), "profile", "docker-compose.yml")


class TestUpDown(unittest.TestCase, RunSubprocessMixin):
    def tearDown(self) -> None:
        """
        Ensures that the services within the "profile compose file" are removed between each test
        case.
        """
        # run the test case

        down_cmd = [
            "coverage",
            "run",
            podman_compose_path(),
            "--profile",
            "profile-1",
            "--profile",
            "profile-2",
            "-f",
            profile_compose_file(),
            "down",
        ]
        self.run_subprocess(down_cmd)

    @parameterized.expand(
        [
            (
                ["--profile", "profile-1", "up", "-d"],
                {"default-service": True, "service-1": True, "service-2": False},
            ),
            (
                ["--profile", "profile-2", "up", "-d"],
                {"default-service": True, "service-1": False, "service-2": True},
            ),
            (
                ["--profile", "profile-1", "--profile", "profile-2", "up", "-d"],
                {"default-service": True, "service-1": True, "service-2": True},
            ),
        ],
    )
    def test_up(self, profiles: List[str], expected_services: dict) -> None:
        up_cmd = [
            "coverage",
            "run",
            podman_compose_path(),
            "-f",
            profile_compose_file(),
        ]
        up_cmd.extend(profiles)

        self.run_subprocess_assert_returncode(up_cmd)

        check_cmd = [
            "podman",
            "container",
            "ps",
            "--format",
            '"{{.Names}}"',
        ]
        out, _ = self.run_subprocess_assert_returncode(check_cmd)

        self.assertEqual(len(expected_services), 3)
        actual_output = out.decode("utf-8")

        actual_services = {}
        for service, _ in expected_services.items():
            actual_services[service] = service in actual_output

        self.assertEqual(expected_services, actual_services)

    @parameterized.expand(
        [
            (
                ["--profile", "profile-1"],
                "profile-2",
                {"default-service": True, "service-1": True, "service-2": True},
            ),
            (
                [],
                "profile-1,profile-2",
                {"default-service": True, "service-1": True, "service-2": True},
            ),
            (
                [],
                "profile-1, profile-2",
                {"default-service": True, "service-1": True, "service-2": True},
            ),
            (
                [],
                "",
                {"default-service": True, "service-1": False, "service-2": False},
            ),
            (
                [],
                ",",
                {"default-service": True, "service-1": False, "service-2": False},
            ),
        ],
    )
    def test_up_with_compose_profiles_env(
        self, profiles: List[str], compose_profiles: str, expected_services: dict
    ) -> None:
        """
        Tests the `up` command when the `COMPOSE_PROFILES` environment variable is set.
        """
        up_cmd = [
            "coverage",
            "run",
            podman_compose_path(),
            "-f",
            profile_compose_file(),
        ]
        up_cmd.extend(profiles)
        up_cmd.extend(["up", "-d"])

        env = os.environ.copy()
        env["COMPOSE_PROFILES"] = compose_profiles

        self.run_subprocess_assert_returncode(up_cmd, env=env)

        check_cmd = [
            "podman",
            "container",
            "ps",
            "--format",
            '"{{.Names}}"',
        ]
        out, _ = self.run_subprocess_assert_returncode(check_cmd)

        self.assertEqual(len(expected_services), 3)
        actual_output = out.decode("utf-8")

        actual_services = {}
        for service, _ in expected_services.items():
            actual_services[service] = service in actual_output

        self.assertEqual(expected_services, actual_services)



# SPDX-License-Identifier: GPL-2.0

import os
import unittest

from parameterized import parameterized

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path():
    """ "Returns the path to the compose file used for this test module"""
    base_path = os.path.join(test_path(), "build/git_url_context")
    return os.path.join(base_path, "docker-compose.yml")


class TestComposeBuildGitUrlAsContext(unittest.TestCase, RunSubprocessMixin):
    @parameterized.expand([
        ("git_url_context_test_context_1", "data_1.txt", b'test1\r\n'),
        ("git_url_context_test_context_1", "data_2.txt", b'test2\r\n'),
        ("git_url_context_test_context_inline_1", "data_1.txt", b'test1\r\n'),
        ("git_url_context_test_context_inline_1", "data_2.txt", b'test2\r\n'),
    ])
    def test_build_git_url_as_context(self, container_name, file_name, output):
        # test if container can access specific files from git repository when git url is used as
        # a build context
        try:
            out, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "up",
                "-d",
            ])

            out, _ = self.run_subprocess_assert_returncode([
                "podman",
                "exec",
                "-ti",
                f"{container_name}",
                "sh",
                "-c",
                f"cat {file_name}",
            ])
            self.assertEqual(out, output)
        finally:
            out, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
                "-t",
                "0",
            ])



# SPDX-License-Identifier: GPL-2.0

import json
import os
import unittest

import requests

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path() -> str:
    return os.path.join(os.path.join(test_path(), "nets_test2"), "docker-compose.yml")


class TestComposeNetsTest2(unittest.TestCase, RunSubprocessMixin):
    # test if port mapping works as expected with networks top-level element
    def test_nets_test2(self) -> None:
        try:
            self.run_subprocess_assert_returncode(
                [
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path(),
                    "up",
                    "-d",
                ],
            )
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "ps",
            ])
            self.assertIn(b"nets_test2_web1_1", output)
            self.assertIn(b"nets_test2_web2_1", output)

            response = requests.get('http://localhost:8001/index.txt')
            self.assertTrue(response.ok)
            self.assertEqual(response.text, "test1\n")

            response = requests.get('http://localhost:8002/index.txt')
            self.assertTrue(response.ok)
            self.assertEqual(response.text, "test2\n")

            # inspect 1st container
            output, _ = self.run_subprocess_assert_returncode([
                "podman",
                "inspect",
                "nets_test2_web1_1",
            ])
            container_info = json.loads(output.decode('utf-8'))[0]

            # check if network got specific name from networks top-level element
            self.assertEqual(
                list(container_info["NetworkSettings"]["Networks"].keys())[0], "nets_test2_mystack"
            )

            # check if Host port is the same as prodvided by the service port
            self.assertIsNotNone(container_info['NetworkSettings']["Ports"].get("8001/tcp", None))
            self.assertGreater(len(container_info['NetworkSettings']["Ports"]["8001/tcp"]), 0)
            self.assertIsNotNone(
                container_info['NetworkSettings']["Ports"]["8001/tcp"][0].get("HostPort", None)
            )
            self.assertEqual(
                container_info['NetworkSettings']["Ports"]["8001/tcp"][0]["HostPort"], "8001"
            )

            self.assertEqual(container_info["Config"]["Hostname"], "web1")

            # inspect 2nd container
            output, _ = self.run_subprocess_assert_returncode([
                "podman",
                "inspect",
                "nets_test2_web2_1",
            ])
            container_info = json.loads(output.decode('utf-8'))[0]

            self.assertEqual(
                list(container_info["NetworkSettings"]["Networks"].keys())[0], "nets_test2_mystack"
            )

            self.assertIsNotNone(container_info['NetworkSettings']["Ports"].get("8001/tcp", None))
            self.assertGreater(len(container_info['NetworkSettings']["Ports"]["8001/tcp"]), 0)
            self.assertIsNotNone(
                container_info['NetworkSettings']["Ports"]["8001/tcp"][0].get("HostPort", None)
            )
            self.assertEqual(
                container_info['NetworkSettings']["Ports"]["8001/tcp"][0]["HostPort"], "8002"
            )

            self.assertEqual(container_info["Config"]["Hostname"], "web2")
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
                "-t",
                "0",
            ])



# SPDX-License-Identifier: GPL-2.0

import os
import shutil
import subprocess
import unittest
from typing import Any
from unittest import mock

from parameterized import parameterized

from podman_compose import PodmanCompose
from podman_compose import PodmanComposeError
from podman_compose import container_to_args


def create_compose_mock(project_name: str = "test_project_name") -> PodmanCompose:
    compose = mock.Mock()
    compose.project_name = project_name
    compose.dirname = "test_dirname"
    compose.container_names_by_service.get = mock.Mock(return_value=None)
    compose.prefer_volume_over_mount = False
    compose.default_net = None
    compose.networks = {}
    compose.x_podman = {}
    compose.join_name_parts = mock.Mock(side_effect=lambda *args: '_'.join(args))
    compose.format_name = mock.Mock(side_effect=lambda *args: '_'.join([project_name, *args]))

    async def podman_output(*args: Any, **kwargs: Any) -> None:
        pass

    compose.podman.output = mock.Mock(side_effect=podman_output)
    return compose


def get_minimal_container() -> dict[str, Any]:
    return {
        "name": "project_name_service_name1",
        "service_name": "service_name",
        "image": "busybox",
    }


def get_test_file_path(rel_path: str) -> str:
    repo_root = os.path.dirname(os.path.dirname(os.path.dirname(__file__)))
    return os.path.realpath(os.path.join(repo_root, rel_path))


class TestContainerToArgs(unittest.IsolatedAsyncioTestCase):
    async def test_external_network_missing_raises_user_friendly_error(self) -> None:
        c = create_compose_mock()
        c.default_net = "external_net"
        c.networks = {
            "external_net": {
                "external": True,
                "name": "missing-external-network",
            }
        }

        async def podman_output(*args: Any, **kwargs: Any) -> None:
            if args[2] == ["exists", "missing-external-network"]:
                raise subprocess.CalledProcessError(1, "podman network exists")

        setattr(c.podman, "output", mock.Mock(side_effect=podman_output))
        cnt = get_minimal_container()

        with self.assertRaisesRegex(
            PodmanComposeError,
            r"External network \[missing-external-network\] does not exist\.",
        ):
            await container_to_args(c, cnt)

    async def test_minimal(self) -> None:
        c = create_compose_mock()

        cnt = get_minimal_container()

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "busybox",
            ],
        )

    async def test_runtime(self) -> None:
        c = create_compose_mock()

        cnt = get_minimal_container()
        cnt["runtime"] = "runsc"

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--runtime",
                "runsc",
                "busybox",
            ],
        )

    async def test_sysctl_list(self) -> None:
        c = create_compose_mock()

        cnt = get_minimal_container()
        cnt["sysctls"] = [
            "net.core.somaxconn=1024",
            "net.ipv4.tcp_syncookies=0",
        ]

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--sysctl",
                "net.core.somaxconn=1024",
                "--sysctl",
                "net.ipv4.tcp_syncookies=0",
                "busybox",
            ],
        )

    async def test_sysctl_map(self) -> None:
        c = create_compose_mock()

        cnt = get_minimal_container()
        cnt["sysctls"] = {
            "net.core.somaxconn": 1024,
            "net.ipv4.tcp_syncookies": 0,
        }

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--sysctl",
                "net.core.somaxconn=1024",
                "--sysctl",
                "net.ipv4.tcp_syncookies=0",
                "busybox",
            ],
        )

    async def test_sysctl_wrong_type(self) -> None:
        c = create_compose_mock()
        cnt = get_minimal_container()

        # check whether wrong types are correctly rejected
        for wrong_type in [True, 0, 0.0, "wrong", ()]:
            with self.assertRaises(TypeError):
                cnt["sysctls"] = wrong_type
                await container_to_args(c, cnt)

    async def test_pid(self) -> None:
        c = create_compose_mock()
        cnt = get_minimal_container()

        cnt["pid"] = "host"

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--pid",
                "host",
                "busybox",
            ],
        )

    async def test_http_proxy(self) -> None:
        c = create_compose_mock()

        cnt = get_minimal_container()
        cnt["http_proxy"] = False

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--http-proxy=false",
                "--network=bridge:alias=service_name",
                "busybox",
            ],
        )

    async def test_uidmaps_extension_old_path(self) -> None:
        c = create_compose_mock()

        cnt = get_minimal_container()
        cnt['x-podman'] = {'uidmaps': ['1000:1000:1']}

        with self.assertRaises(ValueError):
            await container_to_args(c, cnt)

    async def test_uidmaps_extension(self) -> None:
        c = create_compose_mock()

        cnt = get_minimal_container()
        cnt['x-podman.uidmaps'] = ['1000:1000:1', '1001:1001:2']

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                '--uidmap',
                '1000:1000:1',
                '--uidmap',
                '1001:1001:2',
                "busybox",
            ],
        )

    async def test_gidmaps_extension(self) -> None:
        c = create_compose_mock()

        cnt = get_minimal_container()
        cnt['x-podman.gidmaps'] = ['1000:1000:1', '1001:1001:2']

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                '--gidmap',
                '1000:1000:1',
                '--gidmap',
                '1001:1001:2',
                "busybox",
            ],
        )

    async def test_rootfs_extension(self) -> None:
        c = create_compose_mock()

        cnt = get_minimal_container()
        del cnt["image"]
        cnt["x-podman.rootfs"] = "/path/to/rootfs"

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--rootfs",
                "/path/to/rootfs",
            ],
        )

    async def test_no_hosts_extension(self) -> None:
        c = create_compose_mock()

        cnt = get_minimal_container()
        cnt["x-podman.no_hosts"] = True

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--no-hosts",
                "busybox",
            ],
        )

    @parameterized.expand([
        # short syntax: only take this specific environment variable value from .env file
        ("use_env_var_from_default_env_file_short_syntax", ["ZZVAR1"], "ZZVAR1=TEST1"),
        # long syntax: environment variable value from .env file is taken through variable
        # interpolation
        # only the value required in 'environment:' compose file is sent to containers
        # environment
        ("use_env_var_from_default_env_file_long_syntax", ["ZZVAR1=TEST1"], "ZZVAR1=TEST1"),
        # "environment:" section in compose file overrides environment variable value from .env file
        (
            "use_env_var_from_default_env_file_override_value",
            ["ZZVAR1=NEW_TEST1"],
            "ZZVAR1=NEW_TEST1",
        ),
    ])
    async def test_env_file(self, test_name: str, cnt_env: list, expected_var: str) -> None:
        c = create_compose_mock()
        # environment variables were set in .env file
        c.environ = {"ZZVAR1": "TEST1", "ZZVAR2": "TEST2"}

        cnt = get_minimal_container()
        cnt["environment"] = cnt_env

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "-e",
                f"{expected_var}",
                "--network=bridge:alias=service_name",
                "busybox",
            ],
        )

    async def test_env_file_str(self) -> None:
        c = create_compose_mock()

        cnt = get_minimal_container()
        env_file = get_test_file_path('tests/integration/env_file_tests/env-files/project-1.env')
        cnt['env_file'] = env_file

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "-e",
                "ZZVAR1=podman-rocks-123",
                "-e",
                "ZZVAR2=podman-rocks-124",
                "-e",
                "ZZVAR3=podman-rocks-125",
                "--network=bridge:alias=service_name",
                "busybox",
            ],
        )

    async def test_env_file_str_not_exists(self) -> None:
        c = create_compose_mock()

        cnt = get_minimal_container()
        cnt['env_file'] = 'notexists'

        with self.assertRaises(ValueError):
            await container_to_args(c, cnt)

    async def test_env_file_str_array_one_path(self) -> None:
        c = create_compose_mock()

        cnt = get_minimal_container()
        env_file = get_test_file_path('tests/integration/env_file_tests/env-files/project-1.env')
        cnt['env_file'] = [env_file]

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "-e",
                "ZZVAR1=podman-rocks-123",
                "-e",
                "ZZVAR2=podman-rocks-124",
                "-e",
                "ZZVAR3=podman-rocks-125",
                "--network=bridge:alias=service_name",
                "busybox",
            ],
        )

    async def test_env_file_str_array_two_paths(self) -> None:
        c = create_compose_mock()

        cnt = get_minimal_container()
        env_file = get_test_file_path('tests/integration/env_file_tests/env-files/project-1.env')
        env_file_2 = get_test_file_path('tests/integration/env_file_tests/env-files/project-2.env')
        cnt['env_file'] = [env_file, env_file_2]

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "-e",
                "ZZVAR1=podman-rocks-123",
                "-e",
                "ZZVAR2=podman-rocks-124",
                "-e",
                "ZZVAR3=podman-rocks-125",
                "-e",
                "ZZVAR1=podman-rocks-223",
                "-e",
                "ZZVAR2=podman-rocks-224",
                "--network=bridge:alias=service_name",
                "busybox",
            ],
        )

    async def test_env_file_obj_required(self) -> None:
        c = create_compose_mock()

        cnt = get_minimal_container()
        env_file = get_test_file_path('tests/integration/env_file_tests/env-files/project-1.env')
        cnt['env_file'] = {'path': env_file, 'required': True}

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "-e",
                "ZZVAR1=podman-rocks-123",
                "-e",
                "ZZVAR2=podman-rocks-124",
                "-e",
                "ZZVAR3=podman-rocks-125",
                "--network=bridge:alias=service_name",
                "busybox",
            ],
        )

    async def test_env_file_obj_required_non_existent_path(self) -> None:
        c = create_compose_mock()

        cnt = get_minimal_container()
        cnt['env_file'] = {'path': 'not-exists', 'required': True}

        with self.assertRaises(ValueError):
            await container_to_args(c, cnt)

    async def test_env_file_obj_optional(self) -> None:
        c = create_compose_mock()

        cnt = get_minimal_container()
        cnt['env_file'] = {'path': 'not-exists', 'required': False}

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "busybox",
            ],
        )

    async def test_gpu_count_all(self) -> None:
        c = create_compose_mock()

        cnt = get_minimal_container()
        cnt["command"] = ["nvidia-smi"]
        cnt["deploy"] = {"resources": {"reservations": {"devices": [{}]}}}

        cnt["deploy"]["resources"]["reservations"]["devices"][0] = {
            "driver": "nvidia",
            "count": "all",
            "capabilities": ["gpu"],
        }

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--device",
                "nvidia.com/gpu=all",
                "--security-opt=label=disable",
                "busybox",
                "nvidia-smi",
            ],
        )

    async def test_gpu_count_specific(self) -> None:
        c = create_compose_mock()

        cnt = get_minimal_container()
        cnt["command"] = ["nvidia-smi"]
        cnt["deploy"] = {
            "resources": {
                "reservations": {
                    "devices": [
                        {
                            "driver": "nvidia",
                            "count": 2,
                            "capabilities": ["gpu"],
                        }
                    ]
                }
            }
        }

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--device",
                "nvidia.com/gpu=0",
                "--device",
                "nvidia.com/gpu=1",
                "--security-opt=label=disable",
                "busybox",
                "nvidia-smi",
            ],
        )

    async def test_gpu_device_ids_all(self) -> None:
        c = create_compose_mock()

        cnt = get_minimal_container()
        cnt["command"] = ["nvidia-smi"]
        cnt["deploy"] = {
            "resources": {
                "reservations": {
                    "devices": [
                        {
                            "driver": "nvidia",
                            "device_ids": "all",
                            "capabilities": ["gpu"],
                        }
                    ]
                }
            }
        }

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--device",
                "nvidia.com/gpu=all",
                "--security-opt=label=disable",
                "busybox",
                "nvidia-smi",
            ],
        )

    async def test_gpu_device_ids_specific(self) -> None:
        c = create_compose_mock()

        cnt = get_minimal_container()
        cnt["command"] = ["nvidia-smi"]
        cnt["deploy"] = {
            "resources": {
                "reservations": {
                    "devices": [
                        {
                            "driver": "nvidia",
                            "device_ids": [1, 3],
                            "capabilities": ["gpu"],
                        }
                    ]
                }
            }
        }

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--device",
                "nvidia.com/gpu=1",
                "--device",
                "nvidia.com/gpu=3",
                "--security-opt=label=disable",
                "busybox",
                "nvidia-smi",
            ],
        )

    @parameterized.expand([
        (
            False,
            "z",
            [
                "--mount",
                f"type=bind,source={get_test_file_path('test_dirname/foo')},destination=/mnt,z",
            ],
        ),
        (
            False,
            "Z",
            [
                "--mount",
                f"type=bind,source={get_test_file_path('test_dirname/foo')},destination=/mnt,Z",
            ],
        ),
        (True, "z", ["-v", f"{get_test_file_path('test_dirname/foo')}:/mnt:z"]),
        (True, "Z", ["-v", f"{get_test_file_path('test_dirname/foo')}:/mnt:Z"]),
    ])
    async def test_selinux_volume(
        self, prefer_volume: bool, selinux_type: str, expected_additional_args: list
    ) -> None:
        c = create_compose_mock()
        c.prefer_volume_over_mount = prefer_volume

        cnt = get_minimal_container()

        # This is supposed to happen during `_parse_compose_file`
        # but that is probably getting skipped during testing
        cnt["_service"] = cnt["service_name"]

        cnt["volumes"] = [
            {
                "type": "bind",
                "source": "./foo",
                "target": "/mnt",
                "bind": {
                    "selinux": selinux_type,
                },
            }
        ]

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                *expected_additional_args,
                "--network=bridge:alias=service_name",
                "busybox",
            ],
        )

    async def test_volumes_glob_mount_source(self) -> None:
        c = create_compose_mock()
        cnt = get_minimal_container()

        # This is supposed to happen during `_parse_compose_file`
        # but that is probably getting skipped during testing
        cnt["_service"] = cnt["service_name"]

        cnt["volumes"] = [
            {
                "type": "glob",
                "source": f"{get_test_file_path('test_dirname/foo')}/*.ext",
                "target": "/mnt",
            }
        ]
        expected_additional_args = [
            "--mount",
            (
                "type=glob,source="
                f"{get_test_file_path('test_dirname/foo') + '/*.ext'},"
                "destination=/mnt"
            ),
        ]
        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                *expected_additional_args,
                "--network=bridge:alias=service_name",
                "busybox",
            ],
        )

    @parameterized.expand([
        (
            "absolute_path",
            get_test_file_path('test_dirname/foo'),
            [
                "--mount",
                f"type=bind,source={get_test_file_path('test_dirname/foo')},destination=/mnt",
            ],
        ),
        (
            "relative_path",
            './foo',
            [
                "--mount",
                f"type=bind,source={get_test_file_path('test_dirname/foo')},destination=/mnt",
            ],
        ),
        (
            "home_dir",
            '~/test_dirname/foo',
            [
                "--mount",
                f"type=bind,source={os.path.expanduser('~/test_dirname/foo')},destination=/mnt",
            ],
        ),
    ])
    async def test_volumes_bind_mount_source(
        self, test_name: str, mount_source: str, expected_additional_args: list
    ) -> None:
        c = create_compose_mock()
        cnt = get_minimal_container()

        # This is supposed to happen during `_parse_compose_file`
        # but that is probably getting skipped during testing
        cnt["_service"] = cnt["service_name"]

        cnt["volumes"] = [
            {
                "type": "bind",
                "source": f"{mount_source}",
                "target": "/mnt",
            }
        ]

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                *expected_additional_args,
                "--network=bridge:alias=service_name",
                "busybox",
            ],
        )

    @parameterized.expand([
        (
            "without_subpath",
            {},
            "type=image,source=example:latest,destination=/mnt/example",
        ),
        (
            "with_subpath",
            {"image": {"subpath": "path/to/image/folder"}},
            "type=image,source=example:latest,destination=/mnt/example,subpath=path/to/image/folder",
        ),
    ])
    async def test_volumes_image_mount(
        self, test_name: str, image_opts: dict, expected_mount_arg: str
    ) -> None:
        c = create_compose_mock()
        cnt = get_minimal_container()
        cnt["_service"] = cnt["service_name"]

        cnt["volumes"] = [
            {
                "type": "image",
                "source": "example:latest",
                "target": "/mnt/example",
                **image_opts,
            },
        ]

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--mount",
                expected_mount_arg,
                "--network=bridge:alias=service_name",
                "busybox",
            ],
        )

    @parameterized.expand([
        (
            "without_subpath",
            {},
            "type=volume,source=volname,destination=/mnt/example",
        ),
        (
            "with_subpath",
            {"volume": {"subpath": "path/to/image/folder"}},
            "type=volume,source=volname,destination=/mnt/example,subpath=path/to/image/folder",
        ),
    ])
    async def test_volumes_mount(
        self, test_name: str, volume_opts: dict, expected_mount_arg: str
    ) -> None:
        c = create_compose_mock()
        c.vols = {"volname": {"name": "volname"}}

        cnt = get_minimal_container()
        cnt["_service"] = cnt["service_name"]

        cnt["volumes"] = [
            {
                "type": "volume",
                "source": "volname",
                "target": "/mnt/example",
                **volume_opts,
            },
        ]

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--mount",
                expected_mount_arg,
                "--network=bridge:alias=service_name",
                "busybox",
            ],
        )

    @parameterized.expand([
        (
            "create_host_path_set_to_true",
            {"bind": {"create_host_path": True}},
        ),
        (
            "create_host_path_default_true",
            {},
        ),
    ])
    async def test_volumes_bind_mount_create_source_dir(self, test_name: str, bind: dict) -> None:
        # creates a missing source dir
        c = create_compose_mock()
        c.prefer_volume_over_mount = True
        cnt = get_minimal_container()

        cnt["_service"] = cnt["service_name"]

        volume_info = {
            "type": "bind",
            "source": "./not_exists/foo",
            "target": "/mnt",
        }
        volume_info.update(bind)
        cnt["volumes"] = [
            volume_info,
        ]

        args = await container_to_args(c, cnt)

        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "-v",
                f"{get_test_file_path('./test_dirname/not_exists/foo')}:/mnt",
                "--network=bridge:alias=service_name",
                "busybox",
            ],
        )
        dir_path = get_test_file_path('./test_dirname/not_exists/foo')
        shutil.rmtree(dir_path)

    # throws an error as the source path does not exist and its creation was suppressed with the
    # create_host_path = False option
    async def test_volumes_bind_mount_source_does_not_exist(self) -> None:
        c = create_compose_mock()
        c.prefer_volume_over_mount = True
        cnt = get_minimal_container()

        cnt["_service"] = cnt["service_name"]

        cnt["volumes"] = [
            {
                "type": "bind",
                "source": "./not_exists/foo",
                "target": "/mnt",
                "bind": {"create_host_path": False},
            }
        ]

        with self.assertRaises(ValueError):
            await container_to_args(c, cnt)

    @parameterized.expand([
        ("not_compat", False, "test_project_name", "test_project_name_network1"),
        ("compat_no_dash", True, "test_project_name", "test_project_name_network1"),
        ("compat_dash", True, "test_project-name", "test_projectname_network1"),
    ])
    async def test_network_default_name(
        self, name: str, is_compat: bool, project_name: str, expected_network_name: str
    ) -> None:
        c = create_compose_mock(project_name)
        c.x_podman = {PodmanCompose.XPodmanSettingKey.DEFAULT_NET_NAME_COMPAT: is_compat}
        c.networks = {'network1': {}}

        cnt = get_minimal_container()
        cnt['networks'] = ['network1']

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                f"--network={expected_network_name}:alias=service_name",
                "busybox",
            ],
        )

    async def test_device(self) -> None:
        c = create_compose_mock()
        cnt = get_minimal_container()

        cnt['devices'] = ['/dev/ttyS0']
        cnt['device_cgroup_rules'] = ['c 100:200 rwm']

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--device",
                "/dev/ttyS0",
                "--device-cgroup-rule",
                "c 100:200 rwm",
                "--network=bridge:alias=service_name",
                "busybox",
            ],
        )

    async def test_cpuset(self) -> None:
        c = create_compose_mock()
        cnt = get_minimal_container()
        cnt["cpuset"] = "0-1"

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--cpuset-cpus",
                "0-1",
                "busybox",
            ],
        )

    async def test_pids_limit_container_level(self) -> None:
        c = create_compose_mock()
        cnt = get_minimal_container()
        cnt["pids_limit"] = 100

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--pids-limit",
                "100",
                "busybox",
            ],
        )

    async def test_pids_limit_deploy_section(self) -> None:
        c = create_compose_mock()
        cnt = get_minimal_container()
        cnt["deploy"] = {"resources": {"limits": {"pids": 100}}}

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--pids-limit",
                "100",
                "busybox",
            ],
        )

    async def test_pids_limit_both_same(self) -> None:
        c = create_compose_mock()
        cnt = get_minimal_container()
        cnt["pids_limit"] = 100
        cnt["deploy"] = {"resources": {"limits": {"pids": 100}}}

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--pids-limit",
                "100",
                "busybox",
            ],
        )

    async def test_pids_limit_both_different(self) -> None:
        c = create_compose_mock()
        cnt = get_minimal_container()
        cnt["pids_limit"] = 100
        cnt["deploy"] = {"resources": {"limits": {"pids": 200}}}

        with self.assertRaises(ValueError):
            await container_to_args(c, cnt)

    async def test_healthcheck_string(self) -> None:
        c = create_compose_mock()
        cnt = get_minimal_container()
        cnt["healthcheck"] = {
            "test": "cmd arg1 arg2",
        }

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--health-cmd",
                '["CMD-SHELL", "cmd arg1 arg2"]',
                "busybox",
            ],
        )

    async def test_healthcheck_cmd_args(self) -> None:
        c = create_compose_mock()
        cnt = get_minimal_container()
        cnt["healthcheck"] = {
            "test": ["CMD", "cmd", "arg1", "arg2"],
        }

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--health-cmd",
                '["cmd", "arg1", "arg2"]',
                "busybox",
            ],
        )

    async def test_healthcheck_cmd_shell(self) -> None:
        c = create_compose_mock()
        cnt = get_minimal_container()
        cnt["healthcheck"] = {
            "test": ["CMD-SHELL", "cmd arg1 arg2"],
        }

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--health-cmd",
                '["cmd arg1 arg2"]',
                "busybox",
            ],
        )

    async def test_healthcheck_disable(self) -> None:
        c = create_compose_mock()
        cnt = get_minimal_container()
        cnt["healthcheck"] = {
            "test": "cmd arg1 arg2",
            "disable": "true",
        }

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--no-healthcheck",
                "busybox",
            ],
        )

    async def test_healthcheck_cmd_shell_error(self) -> None:
        c = create_compose_mock()
        cnt = get_minimal_container()
        cnt["healthcheck"] = {
            "test": ["CMD-SHELL", "cmd arg1", "arg2"],
        }

        with self.assertRaisesRegex(ValueError, "'CMD-SHELL' takes a single string after it"):
            await container_to_args(c, cnt)

    async def test_unknown_healthcheck_test_type(self) -> None:
        c = create_compose_mock()
        cnt = get_minimal_container()
        cnt["healthcheck"] = {
            "test": ["TEST", "arg1", "arg2"],
        }
        with self.assertRaises(ValueError) as error_msg:
            await container_to_args(c, cnt)

        expected = "unknown healthcheck test type [TEST], expecting NONE, CMD or CMD-SHELL."
        self.assertEqual(expected, str(error_msg.exception))

    async def test_healthcheck_test_not_string_or_list(self) -> None:
        c = create_compose_mock()
        cnt = get_minimal_container()
        cnt["healthcheck"] = {
            "test": 2,
        }

        with self.assertRaisesRegex(ValueError, "'healthcheck.test' either a string or a list"):
            await container_to_args(c, cnt)

    async def test_healthcheck_not_a_dict_error(self) -> None:
        c = create_compose_mock()
        cnt = get_minimal_container()
        cnt["healthcheck"] = []

        with self.assertRaisesRegex(ValueError, "'healthcheck' must be a key-value mapping"):
            await container_to_args(c, cnt)

    async def test_healthcheck_test_is_missing(self) -> None:
        c = create_compose_mock()
        cnt = get_minimal_container()
        cnt["healthcheck"] = {
            "interval": "1m",
            "retries": "3",
        }

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                '--health-interval',
                '1m',
                '--health-retries',
                '3',
                "busybox",
            ],
        )

    async def test_healthcheck_options(self) -> None:
        c = create_compose_mock()
        cnt = get_minimal_container()
        cnt["healthcheck"] = {
            "test": ["CMD", "cmd", "arg1", "arg2"],
            "interval": "1m",
            "timeout": "10s",
            "retries": "3",
            "start_period": "5s",
            "start_interval": "6s",
        }

        args = await container_to_args(c, cnt)
        self.assertEqual(
            args,
            [
                "--name=project_name_service_name1",
                "-d",
                "--network=bridge:alias=service_name",
                "--health-cmd",
                '["cmd", "arg1", "arg2"]',
                '--health-interval',
                '1m',
                '--health-timeout',
                '10s',
                '--health-start-period',
                '5s',
                '--health-startup-interval',
                '6s',
                '--health-retries',
                '3',
                "busybox",
            ],
        )



# SPDX-License-Identifier: GPL-2.0

import os
import unittest

from parameterized import parameterized

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


class TestComposeNameSeparatorCompat(unittest.TestCase, RunSubprocessMixin):
    @parameterized.expand([
        ('default', {}, '_'),
        ('default', {'PODMAN_COMPOSE_NAME_SEPARATOR_COMPAT': '1'}, '-'),
        ('default', {'PODMAN_COMPOSE_DOCKER_COMPOSE_COMPAT': '1'}, '-'),
        ('compat', {}, '-'),
        ('compat', {'PODMAN_COMPOSE_NAME_SEPARATOR_COMPAT': '1'}, '-'),
        ('compat', {'PODMAN_COMPOSE_NAME_SEPARATOR_COMPAT': '0'}, '_'),
    ])
    def test_container_name(self, file: str, env: dict[str, str], expected_sep: str) -> None:
        compose_yaml_path = os.path.join(
            test_path(), "name_separator_compat", f"docker-compose_{file}.yaml"
        )

        try:
            self.run_subprocess_assert_returncode(
                [podman_compose_path(), "-f", compose_yaml_path, "up", "-d"],
                env=env,
            )

            container_name_out, _ = self.run_subprocess_assert_returncode(
                [
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path,
                    "ps",
                    "--format",
                    '{{.Names}}',
                ],
                env=env,
            )
            container_name = container_name_out.decode('utf-8').strip()

            expected_container_name = f'name_separator_compat{expected_sep}web{expected_sep}1'

            self.assertEqual(container_name, expected_container_name)
        finally:
            self.run_subprocess_assert_returncode(
                [
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path,
                    "down",
                    "-t",
                    "0",
                ],
                env=env,
            )



# SPDX-License-Identifier: GPL-2.0

import os
import unittest

import requests

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path() -> str:
    return os.path.join(os.path.join(test_path(), "nethost"), "docker-compose.yaml")


class TestComposeNethost(unittest.TestCase, RunSubprocessMixin):
    # check if container listens for http requests and sends response back
    # as network_mode: host allows to connect to container easily
    def test_nethost(self) -> None:
        try:
            self.run_subprocess_assert_returncode(
                [podman_compose_path(), "-f", compose_yaml_path(), "up", "-d"],
            )

            container_id_out, _ = self.run_subprocess_assert_returncode(
                [
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path(),
                    "ps",
                    "--format",
                    '{{.ID}}',
                ],
            )
            container_id = container_id_out.decode('utf-8').split('\n')[0]
            output, _ = self.run_subprocess_assert_returncode(
                [
                    "podman",
                    "exec",
                    "-it",
                    container_id,
                    "sh",
                    "-c",
                    "echo test_123 >> /tmp/test.txt",
                ],
            )
            response = requests.get('http://localhost:8123/test.txt')
            self.assertEqual(response.ok, True)
            self.assertEqual(response.text, "test_123\n")
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
                "-t",
                "0",
            ])



# SPDX-License-Identifier: GPL-2.0

import os
import unittest

from parameterized import parameterized

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path(failure_order: str) -> str:
    return os.path.join(test_path(), "abort", f"docker-compose-fail-{failure_order}.yaml")


class TestComposeAbort(unittest.TestCase, RunSubprocessMixin):
    @parameterized.expand([
        ("exit", "first", 0),
        ("failure", "first", 1),
        ("exit", "second", 0),
        ("failure", "second", 1),
        ("exit", "simultaneous", 0),
        ("failure", "simultaneous", 1),
        ("exit", "none", 0),
        ("failure", "none", 0),
    ])
    def test_abort(self, abort_type: str, failure_order: str, expected_exit_code: int) -> None:
        try:
            self.run_subprocess_assert_returncode(
                [
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path(failure_order),
                    "up",
                    f"--abort-on-container-{abort_type}",
                ],
                expected_exit_code,
            )
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(failure_order),
                "down",
            ])



# SPDX-License-Identifier: GPL-2.0

import os
import unittest

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def bad_compose_yaml_path() -> str:
    base_path = os.path.join(test_path(), "parse-error")
    return os.path.join(base_path, "docker-compose-error.yml")


def good_compose_yaml_path() -> str:
    base_path = os.path.join(test_path(), "parse-error")
    return os.path.join(base_path, "docker-compose.yml")


class TestComposeBuildParseError(unittest.TestCase, RunSubprocessMixin):
    def test_no_error(self) -> None:
        _, err = self.run_subprocess_assert_returncode(
            [podman_compose_path(), "-f", good_compose_yaml_path(), "config"], 0
        )
        self.assertEqual(b"", err)

    def test_simple_parse_error(self) -> None:
        _, err = self.run_subprocess_assert_returncode(
            [podman_compose_path(), "-f", bad_compose_yaml_path(), "config"], 1
        )
        self.assertIn(b"could not find expected ':'", err)
        self.assertNotIn(b"\nTraceback (most recent call last):\n", err)

    def test_verbose_parse_error_contains_stack_trace(self) -> None:
        _, err = self.run_subprocess_assert_returncode(
            [podman_compose_path(), "--verbose", "-f", bad_compose_yaml_path(), "config"], 1
        )
        self.assertIn(b"\nTraceback (most recent call last):\n", err)



# SPDX-License-Identifier: GPL-2.0

import os
import re
import subprocess
import time
import unittest
from datetime import datetime
from datetime import timedelta
from pathlib import Path
from typing import Any
from typing import Optional

from apscheduler.job import Job
from apscheduler.jobstores.base import JobLookupError
from apscheduler.schedulers.background import BackgroundScheduler
from apscheduler.triggers.interval import IntervalTrigger
from packaging import version
from tzlocal import get_localzone

_podman_version = None


def get_podman_version() -> version.Version:
    """
    Return the *packaging.version.Version* object for the podman binary
    found in PATH (cached after the first call). Raise RuntimeError
    if podman is missing or gives unexpected output.
    """
    global _podman_version
    if _podman_version is None:
        try:
            out = subprocess.check_output(
                ["podman", "--version"], text=True, stderr=subprocess.STDOUT
            )
            # ‘podman version 4.5.0’  →  take last token
            raw = out.strip().split()[-1]
            _podman_version = version.parse(raw)
        except Exception as exc:
            raise RuntimeError("cannot determine podman version") from exc
    return _podman_version


def base_path() -> Path:
    """Returns the base path for the project"""
    return Path(__file__).parent.parent.parent


def test_path() -> str:
    """Returns the path to the tests directory"""
    return os.path.join(base_path(), "tests/integration")


def podman_compose_path() -> str:
    """Returns the path to the podman compose script"""
    return os.path.join(base_path(), "podman_compose.py")


def is_systemd_available() -> bool:
    try:
        with open("/proc/1/comm", encoding="utf-8") as fh:
            return fh.read().strip() == "systemd"
    except FileNotFoundError:
        return False


class RunSubprocessMixin:
    def is_debug_enabled(self) -> bool:
        return "TESTS_DEBUG" in os.environ

    def run_subprocess(
        self,
        args: list[str],
        env: dict[str, str] = {},
        timeout: Optional[float] = None,
    ) -> tuple[bytes, bytes, int]:
        begin = time.time()
        if self.is_debug_enabled():
            print("TEST_CALL", args)
        proc = subprocess.Popen(
            args,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            env=os.environ | env,
        )
        out, err = proc.communicate(timeout=timeout)
        if self.is_debug_enabled():
            print("TEST_CALL completed", time.time() - begin)
            print("STDOUT:", out.decode('utf-8'))
            print("STDERR:", err.decode('utf-8'))
        return out, err, proc.returncode

    def run_subprocess_assert_returncode(
        self,
        args: list[str],
        expected_returncode: int = 0,
        env: dict[str, str] = {},
        timeout: Optional[float] = None,
    ) -> tuple[bytes, bytes]:
        out, err, returncode = self.run_subprocess(args, env=env, timeout=timeout)
        decoded_out = out.decode('utf-8')
        decoded_err = err.decode('utf-8')
        self.assertEqual(  # type: ignore[attr-defined]
            returncode,
            expected_returncode,
            f"Invalid return code of process {returncode} != {expected_returncode}\n"
            f"stdout: {decoded_out}\nstderr: {decoded_err}\n",
        )
        return out, err


class PodmanAwareRunSubprocessMixin(RunSubprocessMixin):
    def retrieve_podman_version(self) -> tuple[int, int, int]:
        out, _ = self.run_subprocess_assert_returncode(["podman", "--version"])
        matcher = re.match(r"\D*(\d+)\.(\d+)\.(\d+)", out.decode('utf-8'))
        if matcher:
            major = int(matcher.group(1))
            minor = int(matcher.group(2))
            patch = int(matcher.group(3))
            return (major, minor, patch)
        raise RuntimeError("Unable to retrieve podman version")


# WORKAROUND for https://github.com/containers/podman/issues/28192
# the healthchecks of Podman rely on systemd timers
# in the test environment (GH actions -> container -> Podman) no systemd is available
# therefore, this class ensures that a periodical healthcheck is run by calling
#   podman healthcheck run <container_name>
# with the help of APScheduler
#
# if systemd is available, this class does nothing
class EnsureHealthcheckRun:
    def __init__(
        self, runner: RunSubprocessMixin, test_case: unittest.TestCase, container_name: str
    ) -> None:
        self.__runner: RunSubprocessMixin = runner

        self.__scheduler: Optional[BackgroundScheduler] = None
        self.__healthcheck_job: Optional[Job] = None
        self.__container_name: str = container_name

        # check if systemd is not available
        if not is_systemd_available():
            # initialize APScheduler
            self.__scheduler = BackgroundScheduler()
            self.__scheduler.start()

            # add a clean up to the test case (in case the test fails)
            test_case.addCleanup(self._cleanup_scheduler)

    def _cleanup_scheduler(self) -> None:
        # stop and remove the healtcheck job
        if self.__scheduler is not None:
            try:
                if self.__healthcheck_job is not None:
                    self.__scheduler.remove_job(self.__healthcheck_job.id)
                else:
                    self.__scheduler.remove_all_jobs()
            except JobLookupError:
                # failover
                self.__scheduler.remove_all_jobs()

            # shutdown the scheduler
            self.__scheduler.shutdown(wait=True)

            # prevent this method from being executed twice:
            # after exiting the context manager and during clean up of the test case
            self.__scheduler = None

    def __enter__(self) -> None:
        # start the healthcheck job
        if self.__scheduler is not None:
            self.__healthcheck_job = self.__scheduler.add_job(
                func=self.__runner.run_subprocess,
                # run health checking only for the provided container
                kwargs={"args": ["podman", "healthcheck", "run", f"{self.__container_name}"]},
                trigger=IntervalTrigger(
                    # trigger the healthcheck every 3 seconds (like defined in docker-compose.yml)
                    seconds=3,
                    # run first healthcheck after 3 seconds (initial interval)
                    start_date=datetime.now(get_localzone()) + timedelta(seconds=3),
                    # stop after 21 seconds = 3s (interval) * 6 (retries) + 3s (initial interval)
                    end_date=datetime.now(get_localzone()) + timedelta(seconds=21),
                ),
            )

    def __exit__(self, *_: Any) -> None:
        # clean up after exiting the context manager
        self._cleanup_scheduler()


class ExecutionTime:
    def __init__(
        self,
        min_execution_time: Optional[timedelta] = None,
        max_execution_time: Optional[timedelta] = None,
    ) -> None:
        self.__min_execution_time = min_execution_time
        self.__max_execution_time = max_execution_time

        self.__start_time: Optional[datetime] = None

    def __enter__(self) -> None:
        self.__start_time = datetime.now()

    def __exit__(self, *_: Any) -> None:
        # should normally not happen
        if self.__start_time is None:
            return

        execution_time = datetime.now() - self.__start_time

        if self.__min_execution_time:
            assert execution_time >= self.__min_execution_time
        if self.__max_execution_time:
            assert execution_time <= self.__max_execution_time



# SPDX-License-Identifier: GPL-2.0

import os
import unittest
from pathlib import Path

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path() -> str:
    return os.path.join(os.path.join(test_path(), "extends_w_file_subdir"), "docker-compose.yml")


class TestComposeExtendsWithFileSubdir(unittest.TestCase, RunSubprocessMixin):
    def test_extends_w_file_subdir(self) -> None:  # when file is Dockerfile for building the image
        try:
            self.run_subprocess_assert_returncode(
                [
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path(),
                    "up",
                ],
            )
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "ps",
            ])
            self.assertIn("extends_w_file_subdir_web_1", str(output))
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
            ])

    def test_podman_compose_extends_w_file_subdir(self) -> None:
        """
        Test that podman-compose can execute podman-compose -f <file> up with extended File which
        includes a build context
        :return:
        """
        main_path = Path(__file__).parent.parent.parent.parent

        command_up = [
            "coverage",
            "run",
            str(main_path.joinpath("podman_compose.py")),
            "-f",
            str(
                main_path.joinpath(
                    "tests", "integration", "extends_w_file_subdir", "docker-compose.yml"
                )
            ),
            "up",
            "-d",
        ]

        command_check_container = [
            "coverage",
            "run",
            str(main_path.joinpath("podman_compose.py")),
            "-f",
            str(
                main_path.joinpath(
                    "tests", "integration", "extends_w_file_subdir", "docker-compose.yml"
                )
            ),
            "ps",
            "--format",
            '{{.Image}}',
        ]

        self.run_subprocess_assert_returncode(command_up)
        # check container was created and exists
        out, _ = self.run_subprocess_assert_returncode(command_check_container)
        self.assertEqual(out, b'localhost/subdir_test:me\n')
        # cleanup test image(tags)
        self.run_subprocess_assert_returncode([
            str(main_path.joinpath("podman_compose.py")),
            "-f",
            str(
                main_path.joinpath(
                    "tests", "integration", "extends_w_file_subdir", "docker-compose.yml"
                )
            ),
            "down",
        ])

        self.run_subprocess_assert_returncode([
            "podman",
            "rmi",
            "--force",
            "localhost/subdir_test:me",
        ])

        # check container did not exist anymore
        out, _ = self.run_subprocess_assert_returncode(command_check_container)
        self.assertEqual(out, b'')



# SPDX-License-Identifier: GPL-2.0

import json
import os
import subprocess
import unittest

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


class TestPodmanCompose(unittest.TestCase, RunSubprocessMixin):
    def test_selinux(self) -> None:
        # test if when using volumes type:bind with selinux:z option, container ackquires a
        # respective host:source:z mapping in CreateCommand list
        compose_path = os.path.join(test_path(), "selinux", "docker-compose.yml")
        try:
            # change working directory to where docker-compose.yml file is so that containers can
            # directly access host source file for mounting from that working directory
            subprocess.run(
                [
                    podman_compose_path(),
                    "-f",
                    compose_path,
                    "up",
                    "-d",
                    "container1",
                    "container2",
                ],
                cwd=os.path.join(test_path(), 'selinux'),
            )
            out, _ = self.run_subprocess_assert_returncode([
                "podman",
                "inspect",
                "selinux_container1_1",
            ])
            inspect_out = json.loads(out)
            create_command_list = inspect_out[0].get("Config", []).get("CreateCommand", [])
            host_path = os.path.join(test_path(), "selinux", "host_test_text.txt")
            self.assertIn(f'{host_path}:/test_text.txt:z', create_command_list)

            out, _ = self.run_subprocess_assert_returncode([
                "podman",
                "inspect",
                "selinux_container2_1",
            ])
            inspect_out = json.loads(out)
            create_command_list = inspect_out[0].get("Config", []).get("CreateCommand", [])
            host_path = os.path.join(test_path(), "selinux", "host_test_text.txt")
            self.assertIn(f'{host_path}:/test_text.txt', create_command_list)
        finally:
            out, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_path,
                "down",
                "-t",
                "0",
            ])



# SPDX-License-Identifier: GPL-2.0

import os
import unittest

import requests

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path():
    """ "Returns the path to the compose file used for this test module"""
    base_path = os.path.join(test_path(), "build")
    return os.path.join(base_path, "docker-compose.yml")


class TestComposeBuild(unittest.TestCase, RunSubprocessMixin):
    def test_build(self):
        try:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "build",
                "--no-cache",
            ])

            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "up",
                "-d",
            ])

            request = requests.get('http://localhost:8080/index.txt')
            self.assertEqual(request.status_code, 200)

            alt_request_success = False
            try:
                # FIXME: suspicious behaviour, too often ends up in error
                alt_request = requests.get('http://localhost:8000/index.txt')
                self.assertEqual(alt_request.status_code, 200)
                self.assertIn("ALT buildno=2 port=8000 ", alt_request.text)
                alt_request_success = True
            except requests.exceptions.ConnectionError:
                pass

            if alt_request_success:
                output, _ = self.run_subprocess_assert_returncode([
                    "podman",
                    "inspect",
                    "my-busybox-httpd2",
                ])
                self.assertIn("httpd_port=8000", str(output))
                self.assertIn("buildno=2", str(output))
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
            ])



# SPDX-License-Identifier: GPL-2.0

import os
import unittest

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path() -> str:
    return os.path.join(os.path.join(test_path(), "nets_test_ip"), "docker-compose.yml")


class TestComposeNetsTestIp(unittest.TestCase, RunSubprocessMixin):
    # test if services retain custom ipv4_address and mac_address matching the subnet provided
    # in networks top-level element
    def test_nets_test_ip(self) -> None:
        try:
            self.run_subprocess_assert_returncode(
                [
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path(),
                    "up",
                    "-d",
                ],
            )

            expected_results = [
                (
                    "web1",
                    b"inet 172.19.1.10/24 ",
                    b"link/ether 02:01:01:00:01:01 ",
                    b"inet 172.19.2.10/24 ",
                    b"link/ether 02:01:01:00:02:01 ",
                    b"",
                ),
                ("web2", b"", b"", b"inet 172.19.2.11/24 ", b"", b"link/ether 02:01:01:00:02:02 "),
                ("web3", b"", b"", b"inet 172.19.2.", b"", b""),
                ("web4", b"inet 172.19.1.13/24 ", b"", b"inet 172.19.2.", b"", b""),
            ]

            for (
                service_name,
                shared_network_ip,
                shared_network_mac_address,
                internal_network_ip,
                internal_network_mac_address,
                mac_address,
            ) in expected_results:
                output, _ = self.run_subprocess_assert_returncode([
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path(),
                    "exec",
                    service_name,
                    "ip",
                    "a",
                ])
                self.assertIn(shared_network_ip, output)
                self.assertIn(shared_network_mac_address, output)
                self.assertIn(internal_network_ip, output)
                self.assertIn(internal_network_mac_address, output)
                self.assertIn(mac_address, output)
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
                "-t",
                "0",
            ])



# SPDX-License-Identifier: GPL-2.0

import json
import os
import unittest

from packaging import version

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import get_podman_version


def base_path() -> str:
    """Returns the base path for the project"""
    return os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(__file__))))


def test_path() -> str:
    """Returns the path to the tests directory"""
    return os.path.join(base_path(), "tests/integration")


def podman_compose_path() -> str:
    """Returns the path to the podman compose script"""
    return os.path.join(base_path(), "podman_compose.py")


class TestPodmanComposePodArgs(unittest.TestCase, RunSubprocessMixin):
    def load_pod_info(self, pod_name: str) -> dict:
        output, _ = self.run_subprocess_assert_returncode([
            "podman",
            "pod",
            "inspect",
            pod_name,
        ])
        pod_info = json.loads(output.decode('utf-8'))
        # Podman 5.0 changed pod inspect to always output a list.
        # Check type to support both old and new version.
        if isinstance(pod_info, list):
            return pod_info[0]
        return pod_info

    def run_pod_args_test(self, config: str, args: list, expected: list) -> None:
        """
        Helper to run podman up with a docker-compose.yml config, additional
        (--pod-args) arguments and compare the CreateCommand of the resulting
        pod with an expected value
        """
        pod_name = "pod_" + config
        command_up = (
            [
                "python3",
                os.path.join(base_path(), "podman_compose.py"),
                "-f",
                os.path.join(
                    base_path(),
                    "tests",
                    "integration",
                    "pod_args",
                    config,
                    "docker-compose.yml",
                ),
            ]
            + args
            + [
                "up",
                "--no-start",
            ]
        )

        try:
            self.run_subprocess_assert_returncode(command_up)

            pod_info = self.load_pod_info(pod_name)
            self.assertEqual(
                pod_info['CreateCommand'],
                ["podman", "pod", "create", "--name=" + pod_name] + expected,
            )

        finally:
            command_rm_pod = ["podman", "pod", "rm", pod_name]
            self.run_subprocess_assert_returncode(command_rm_pod)

    def test_x_podman_pod_args_unset_unset(self) -> None:
        """
        Test that podman-compose will use the default pod-args when unset in
        both docker-compose.yml and command line
        """
        self.run_pod_args_test(
            "custom_pod_args_unset",
            [],
            ["--infra=false", "--share="],
        )

    @unittest.skipIf(get_podman_version() >= version.parse("5.0.0"), "Breaks as of podman-5.4.2.")
    def test_x_podman_pod_args_unset_empty(self) -> None:
        """
        Test that podman-compose will use empty pod-args when unset in
        docker-compose.yml and passing an empty value on the command line
        """
        self.run_pod_args_test(
            "custom_pod_args_unset",
            ["--pod-args="],
            [],
        )

    @unittest.skipIf(get_podman_version() >= version.parse("5.0.0"), "Breaks as of podman-5.4.2.")
    def test_x_podman_pod_args_unset_set(self) -> None:
        """
        Test that podman-compose will use the passed pod-args when unset in
        docker-compose.yml and passing a non-empty value on the command line
        """
        self.run_pod_args_test(
            "custom_pod_args_unset",
            ["--pod-args=--infra=false --share= --cpus=1"],
            ["--infra=false", "--share=", "--cpus=1"],
        )

    @unittest.skipIf(get_podman_version() >= version.parse("5.0.0"), "Breaks as of podman-5.4.2.")
    def test_x_podman_pod_args_empty_unset(self) -> None:
        """
        Test that podman-compose will use empty pod-args when set to an
        empty value in docker-compose.yml and unset on the command line
        """
        self.run_pod_args_test(
            "custom_pod_args_empty",
            [],
            [],
        )

    @unittest.skipIf(get_podman_version() >= version.parse("5.0.0"), "Breaks as of podman-5.4.2.")
    def test_x_podman_pod_args_empty_empty(self) -> None:
        """
        Test that podman-compose will use empty pod-args when set to an
        empty value in both docker-compose.yml and command line
        """
        self.run_pod_args_test(
            "custom_pod_args_empty",
            ["--pod-args="],
            [],
        )

    def test_x_podman_pod_args_empty_set(self) -> None:
        """
        Test that podman-compose will use the passed pod-args when set to an
        empty value in docker-compose.yml and passing a non-empty value on the
        command line
        """
        self.run_pod_args_test(
            "custom_pod_args_empty",
            ["--pod-args=--infra=false --share= --cpus=1"],
            ["--infra=false", "--share=", "--cpus=1"],
        )

    def test_x_podman_pod_args_set_unset(self) -> None:
        """
        Test that podman-compose will use the set pod-args when set to a
        non-empty value in docker-compose.yml and unset on the command line
        """
        self.run_pod_args_test(
            "custom_pod_args_set",
            [],
            ["--infra=false", "--share=", "--cpus=2"],
        )

    @unittest.skipIf(get_podman_version() >= version.parse("5.0.0"), "Breaks as of podman-5.4.2.")
    def test_x_podman_pod_args_set_empty(self) -> None:
        """
        Test that podman-compose will use empty pod-args when set to a
        non-empty value in docker-compose.yml and passing an empty value on
        the command line
        """
        self.run_pod_args_test(
            "custom_pod_args_set",
            ["--pod-args="],
            [],
        )

    def test_x_podman_pod_args_set_set(self) -> None:
        """
        Test that podman-compose will use the passed pod-args when set to a
        non-empty value in both docker-compose.yml and command line
        """
        self.run_pod_args_test(
            "custom_pod_args_set",
            ["--pod-args=--infra=false --share= --cpus=1"],
            ["--infra=false", "--share=", "--cpus=1"],
        )



# SPDX-License-Identifier: GPL-2.0

import os
import unittest

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path() -> str:
    return os.path.join(os.path.join(test_path(), "env_tests"), "container-compose.yml")


class TestComposeEnv(unittest.TestCase, RunSubprocessMixin):
    """Test that inline environment variable overrides environment variable from compose file."""

    def test_env(self) -> None:
        try:
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "run",
                "-l",
                "monkey",
                "-e",
                "ZZVAR1=myval2",
                "env-test",
            ])
            self.assertIn("ZZVAR1='myval2'", str(output))
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
            ])

    """
    Tests interpolation of COMPOSE_PROJECT_NAME in the podman-compose config,
    which is different from external environment variables because COMPOSE_PROJECT_NAME
    is a predefined environment variable generated from the `name` value in the top-level
    of the compose.yaml.

    See also
    - https://docs.docker.com/reference/compose-file/interpolation/
    - https://docs.docker.com/reference/compose-file/version-and-name/#name-top-level-element
    - https://docs.docker.com/compose/how-tos/environment-variables/envvars/
    - https://github.com/compose-spec/compose-spec/blob/main/04-version-and-name.md
    """

    def test_project_name(self) -> None:
        try:
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "run",
                "project-name-test",
            ])
            self.assertIn("my-project-name", str(output))
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
            ])

    def test_project_name_override(self) -> None:
        try:
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "run",
                "-e",
                "COMPOSE_PROJECT_NAME=project-name-override",
                "project-name-test",
            ])
            self.assertIn("project-name-override", str(output))
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
            ])

    def test_project_name_interpolation(self) -> None:
        try:
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "run",
                "-e",
                "ZZVAR1=-myval2",
                "project-name-test",
            ])
            self.assertIn("my-project-name-myval2", str(output))
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
            ])



import unittest
from typing import Any

from parameterized import parameterized

from podman_compose import flat_deps


class TestDependsOn(unittest.TestCase):
    @parameterized.expand([
        (
            {
                "service_a": {},
                "service_b": {"depends_on": {"service_a": {"condition": "healthy"}}},
                "service_c": {"depends_on": {"service_b": {"condition": "healthy"}}},
            },
            # dependencies
            {
                "service_a": set(),
                "service_b": set(["service_a"]),
                "service_c": set(["service_a", "service_b"]),
            },
            # dependents
            {
                "service_a": set(["service_b", "service_c"]),
                "service_b": set(["service_c"]),
                "service_c": set(),
            },
        ),
    ])
    def test_flat_deps(
        self,
        services: dict[str, Any],
        deps: dict[str, set[str]],
        dependents: dict[str, set[str]],
    ) -> None:
        flat_deps(services)
        self.assertEqual(
            {
                name: set([x.name for x in value.get("_deps", set())])
                for name, value in services.items()
            },
            deps,
            msg="Dependencies do not match",
        )
        self.assertEqual(
            {
                name: set([x.name for x in value.get("_dependents", set())])
                for name, value in services.items()
            },
            dependents,
            msg="Dependents do not match",
        )



# SPDX-License-Identifier: GPL-2.0

import os
import unittest

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path() -> str:
    return os.path.join(
        test_path(),
        "merge/reset_and_override_tags/reset_tag_attribute/docker-compose.yaml",
    )


class TestComposeResetTagAttribute(unittest.TestCase, RunSubprocessMixin):
    # test if the attribute of the service is correctly reset
    def test_reset_tag_attribute(self) -> None:
        reset_file = os.path.join(
            test_path(),
            "merge/reset_and_override_tags/reset_tag_attribute/docker-compose.reset_attribute.yaml",
        )
        try:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "-f",
                reset_file,
                "up",
            ])

            # the service still exists, but its command attribute was reset in
            # docker-compose.reset_tag_attribute.yaml file and is now empty
            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "-f",
                reset_file,
                "ps",
            ])
            self.assertIn(b"reset_tag_attribute_app_1", output)

            output, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "-f",
                reset_file,
                "logs",
            ])
            self.assertEqual(output, b"")
            # depends_on: !reset null testing: if this test works, depends_on is correctly reset.
            # Otherwise the test would break, since "db" dependency service is not provided
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
            ])



# SPDX-License-Identifier: GPL-2.0

import json
import os
import unittest
from datetime import timedelta
from io import BytesIO
from typing import Final

from packaging import version

from tests.integration.test_utils import EnsureHealthcheckRun
from tests.integration.test_utils import ExecutionTime
from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import get_podman_version
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path

EXECUTION_TIMEOUT: Final[float] = 30


def compose_yaml_path() -> str:
    return os.path.join(test_path(), "wait", "docker-compose.yml")


@unittest.skipIf(
    get_podman_version() < version.parse("4.6.0"), "Wait feature not supported below 4.6.0."
)
class TestComposeWait(unittest.TestCase, RunSubprocessMixin):
    def setUp(self) -> None:
        # build the test image before starting the tests
        # this is not possible in setUpClass method, because the run_subprocess_* methods are not
        # available as classmethods
        self.run_subprocess_assert_returncode([
            podman_compose_path(),
            "-f",
            compose_yaml_path(),
            "build",
        ])

    def _get_health_status(self, container_name: str) -> str:
        output, _ = self.run_subprocess_assert_returncode([
            "podman",
            "inspect",
            container_name,
        ])
        inspect = json.load(BytesIO(output))

        self.assertEqual(len(inspect), 1)
        self.assertIn("State", inspect[0])
        self.assertIn("Health", inspect[0]["State"])
        self.assertIn("Status", inspect[0]["State"]["Health"])

        return inspect[0]["State"]["Health"]["Status"]

    def _is_running(self, container_name: str) -> bool:
        output, _ = self.run_subprocess_assert_returncode([
            "podman",
            "inspect",
            container_name,
        ])
        inspect = json.load(BytesIO(output))

        self.assertEqual(len(inspect), 1)
        self.assertIn("State", inspect[0])
        self.assertIn("Running", inspect[0]["State"])
        return inspect[0]["State"]["Running"]

    def _compose_down(self) -> None:
        self.run_subprocess_assert_returncode([
            podman_compose_path(),
            "-f",
            compose_yaml_path(),
            "down",
            "-t",
            "0",
        ])

    def test_without_wait(self) -> None:
        try:
            with EnsureHealthcheckRun(
                runner=self, test_case=self, container_name="wait_app_health_1"
            ):
                # the execution time of this command must be not more then 10 seconds
                # otherwise this test case makes no sense
                with ExecutionTime(max_execution_time=timedelta(seconds=10)):
                    self.run_subprocess_assert_returncode([
                        podman_compose_path(),
                        "-f",
                        compose_yaml_path(),
                        "up",
                        "-d",
                    ])

                output, _ = self.run_subprocess_assert_returncode([
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path(),
                    "ps",
                ])
                self.assertIn(b"wait_app_health_1", output)
                self.assertIn(b"wait_app_1", output)

                self.assertTrue(self._is_running("wait_app_1"))

                health = self._get_health_status("wait_app_health_1")
                self.assertEqual(health, "starting")
        finally:
            self._compose_down()

    def test_wait(self) -> None:
        try:
            with EnsureHealthcheckRun(
                runner=self, test_case=self, container_name="wait_app_health_1"
            ):
                # the execution time of this command must be at least 10 seconds,
                # because of the sleep command in entrypoint.sh
                with ExecutionTime(min_execution_time=timedelta(seconds=10)):
                    self.run_subprocess_assert_returncode(
                        [
                            podman_compose_path(),
                            "-f",
                            compose_yaml_path(),
                            "up",
                            "-d",
                            "--wait",
                        ],
                        timeout=EXECUTION_TIMEOUT,
                    )

                output, _ = self.run_subprocess_assert_returncode([
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path(),
                    "ps",
                ])
                self.assertIn(b"wait_app_health_1", output)
                self.assertIn(b"wait_app_1", output)

                self.assertTrue(self._is_running("wait_app_1"))

                health = self._get_health_status("wait_app_health_1")
                self.assertEqual(health, "healthy")
        finally:
            self._compose_down()

    def test_wait_with_timeout(self) -> None:
        try:
            with EnsureHealthcheckRun(
                runner=self, test_case=self, container_name="wait_app_health_1"
            ):
                # the execution time of this command must be between 5 and 10 seconds
                with ExecutionTime(
                    min_execution_time=timedelta(seconds=5),
                    max_execution_time=timedelta(seconds=10),
                ):
                    self.run_subprocess_assert_returncode(
                        [
                            podman_compose_path(),
                            "-f",
                            compose_yaml_path(),
                            "up",
                            "-d",
                            "--wait",
                            "--wait-timeout",
                            "5",
                        ],
                        timeout=EXECUTION_TIMEOUT,
                    )

                output, _ = self.run_subprocess_assert_returncode([
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path(),
                    "ps",
                ])
                self.assertIn(b"wait_app_health_1", output)
                self.assertIn(b"wait_app_1", output)

                self.assertTrue(self._is_running("wait_app_1"))

                health = self._get_health_status("wait_app_health_1")
                self.assertEqual(health, "starting")
        finally:
            self._compose_down()

    def test_wait_with_start(self) -> None:
        try:
            with EnsureHealthcheckRun(
                runner=self, test_case=self, container_name="wait_app_health_1"
            ):
                # the execution time of this command must be not more then 10 seconds
                # otherwise this test case makes no sense
                with ExecutionTime(max_execution_time=timedelta(seconds=10)):
                    # podman-compose create does not exist
                    # therefore bring the containers up and kill them immediately again
                    self.run_subprocess_assert_returncode([
                        podman_compose_path(),
                        "-f",
                        compose_yaml_path(),
                        "up",
                        "-d",
                    ])
                    self.run_subprocess_assert_returncode([
                        podman_compose_path(),
                        "-f",
                        compose_yaml_path(),
                        "kill",
                        "--all",
                    ])

                output, _ = self.run_subprocess_assert_returncode([
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path(),
                    "ps",
                ])
                self.assertIn(b"wait_app_health_1", output)
                self.assertIn(b"wait_app_1", output)

                self.assertFalse(self._is_running("wait_app_health_1"))
                self.assertFalse(self._is_running("wait_app_1"))

                # the execution time of this command must be at least 10 seconds,
                # because of the sleep command in entrypoint.sh
                with ExecutionTime(min_execution_time=timedelta(seconds=10)):
                    self.run_subprocess_assert_returncode(
                        [
                            podman_compose_path(),
                            "-f",
                            compose_yaml_path(),
                            "start",
                            "--wait",
                        ],
                        timeout=EXECUTION_TIMEOUT,
                    )

                self.assertTrue(self._is_running("wait_app_1"))

                health = self._get_health_status("wait_app_health_1")
                self.assertEqual(health, "healthy")
        finally:
            self._compose_down()



# SPDX-License-Identifier: GPL-2.0


import os
import time
import unittest

from packaging import version
from parameterized import parameterized

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import get_podman_version
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


class TestLifetime(unittest.TestCase, RunSubprocessMixin):
    def test_up_single_container(self) -> None:
        """Podman compose up should be able to start containers one after another"""

        compose_path = os.path.join(test_path(), "lifetime/up_single_container/docker-compose.yml")

        try:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_path,
                "up",
                "-d",
                "container1",
            ])

            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_path,
                "up",
                "-d",
                "container2",
            ])

            out, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_path,
                "logs",
                "--no-log-prefix",
                "--no-color",
                "container1",
            ])

            self.assertEqual(out, b"test1\n")

            out, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_path,
                "logs",
                "--no-log-prefix",
                "--no-color",
                "container2",
            ])

            self.assertEqual(out, b"test2\n")

        finally:
            out, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_path,
                "down",
            ])

    @parameterized.expand([
        ("no_ports", "up_single_container_many_times"),
        ("with_ports", "up_single_container_many_times_with_ports"),
    ])
    @unittest.skipIf(get_podman_version() >= version.parse("5.0.0"), "Breaks as of podman-5.4.2.")
    def test_up_single_container_many_times(self, name: str, subdir: str) -> None:
        """Podman compose up should be able to start a container many times after it finishes
        running.
        """

        compose_path = os.path.join(test_path(), f"lifetime/{subdir}/docker-compose.yml")

        try:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_path,
                "up",
                "-d",
                "container1",
            ])

            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_path,
                "up",
                "-d",
                "container2",
            ])

            out, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_path,
                "logs",
                "--no-log-prefix",
                "--no-color",
                "container1",
            ])

            self.assertEqual(out, b"test1\n")

            # "restart: always" keeps restarting container until its removal
            out, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_path,
                "logs",
                "--no-log-prefix",
                "--no-color",
                "container2",
            ])

            if not out.startswith(b"test2\ntest2"):
                time.sleep(1)
                out, _ = self.run_subprocess_assert_returncode([
                    podman_compose_path(),
                    "-f",
                    compose_path,
                    "logs",
                    "--no-log-prefix",
                    "--no-color",
                    "container2",
                ])
            self.assertTrue(out.startswith(b"test2\ntest2"))
        finally:
            out, _ = self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_path,
                "down",
                "-t",
                "0",
            ])



# SPDX-License-Identifier: GPL-2.0

import os
import unittest

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


def compose_yaml_path() -> str:
    """ "Returns the path to the compose file used for this test module"""
    base_path = os.path.join(test_path(), "commands_fail_exit_code")
    return os.path.join(base_path, "docker-compose.yml")


class TestComposeCommandsFailExitCodes(unittest.TestCase, RunSubprocessMixin):
    def test_build_command_fail(self) -> None:
        output, error = self.run_subprocess_assert_returncode(
            [
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "build",
                # prevent the successful build from being cached to ensure it runs long enough
                "--no-cache",
            ],
            expected_returncode=1,
        )
        self.assertIn("RUN false", str(output))
        self.assertIn("while running runtime: exit status 1", str(error))

    def test_push_command_fail(self) -> None:
        # test that push command is able to return other than "0" return code
        # "push" command fails due to several steps missing before running it (logging, tagging)
        try:
            output, error = self.run_subprocess_assert_returncode(
                [
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path(),
                    "push",
                    "good",
                ],
                expected_returncode=125,
            )
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
            ])

    def test_pull_command_fail(self) -> None:
        # test that pull command is able to return other than "0" return code
        # "pull" command fails due to missing images or networking errors
        # failure in pulling any one of the images ends up in error code for "pull" command
        try:
            output, error = self.run_subprocess_assert_returncode(
                [
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path(),
                    "pull",
                    "test_pull_bad",
                    "test_pull_good",
                ],
                expected_returncode=125,
            )
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
            ])

    def test_run_command_fail(self) -> None:
        # test that run command is able to return other than "0" return code
        try:
            output, error = self.run_subprocess_assert_returncode(
                [
                    podman_compose_path(),
                    "-f",
                    compose_yaml_path(),
                    "run",
                    "bad",
                ],
                expected_returncode=125,
            )
            self.assertIn("RUN false", str(output))
            self.assertIn("while running runtime: exit status 1", str(error))
        finally:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_yaml_path(),
                "down",
            ])



# SPDX-License-Identifier: GPL-2.0

import unittest

from parameterized import parameterized

from podman_compose import is_context_git_url


class TestIsPathGitUrl(unittest.TestCase):
    @parameterized.expand([
        ("with_url_fragment", "http://host.xz/path/to/repo.git#fragment", True),
        ("suffix_and_prefix", "git://host.xz/path/to/repo.git", True),
        ("empty_url_path", "http://#fragment", True),
        ("no_prefix", "http://host.xz/path/to/repo", True),
        ("wrong_prefix_git", "gitt://host.xz/path/to/repo", False),
        ("wrong_prefix_http", "htt://host.xz/path/to/repo.git", False),
        ("user_path_ending_with_git", "path/to/workdir.git", False),
        ("", "/path/to/workdir.git", False),
        ("", "/path/to:workdir.git", False),
        ("", "/path/to@workdir.git", False),
        ("", "~/path/to/workdir.git", False),
        # many of possible ways git url can look like
        ("", "http://example.com/my-project.git", True),
        ("", "file:///absolute/path/to/my-project.git", True),
        ("", "ssh:user@example.com:my-project", True),
        ("", "git@github.com:user/project.git", True),
        ("", "https://github.com/user/project.git", True),
        ("", "http://github.com/user/project.git", True),
        ("", "git@192.168.101.127:user/project.git", True),
        ("", "https://192.168.101.127/user/project.git", True),
        ("", "http://192.168.101.127/user/project.git", True),
        ("", "ssh://user@host.xz:port/path/to/repo.git/", True),
        ("", "ssh://user@host.xz/path/to/repo.git/", True),
        ("", "ssh://host.xz:port/path/to/repo.git/", True),
        ("", "ssh://host.xz/path/to/repo.git/", True),
        ("", "ssh://user@host.xz/path/to/repo.git/", True),
        ("", "ssh://host.xz/path/to/repo.git/", True),
        ("", "ssh://user@host.xz/~user/path/to/repo.git/", True),
        ("", "ssh://host.xz/~user/path/to/repo.git/", True),
        ("", "ssh://user@host.xz/~/path/to/repo.git", True),
        ("", "ssh://host.xz/~/path/to/repo.git", True),
        ("", "git://host.xz/path/to/repo.git/", True),
        ("", "git://host.xz/~user/path/to/repo.git/", True),
        ("", "http://host.xz/path/to/repo.git/", True),
        ("", "https://host.xz/path/to/repo.git/", True),
        ("", "git@custom-gitlab:my-group/myrepo.git", True),
        ("", "ssh://user@host.xz:port/path/to/repo.git/", True),
        ("", "ssh://user@host.xz/path/to/repo.git/", True),
        ("", "ssh://host.xz:port/path/to/repo.git/", True),
        ("", "ssh://host.xz/path/to/repo.git/", True),
        ("", "ssh://user@host.xz/path/to/repo.git/", True),
        ("", "ssh://host.xz/path/to/repo.git/", True),
        ("", "ssh://user@host.xz/~user/path/to/repo.git/", True),
        ("", "ssh://host.xz/~user/path/to/repo.git/", True),
        ("", "ssh://user@host.xz/~/path/to/repo.git", True),
        ("", "ssh://host.xz/~/path/to/repo.git", True),
        ("", "git://host.xz/path/to/repo.git/", True),
        ("", "git://host.xz/~user/path/to/repo.git/", True),
        ("", "http://host.xz/path/to/repo.git/", True),
        ("", "https://host.xz/path/to/repo.git/", True),
        ("", "ssh:user@example.com:my-project", True),
        ("", "user@host.xz:/path/to/repo.git/", True),
        ("", "host.xz:/path/to/repo.git/", True),
        ("", "host:path/to/repo.git", True),
        ("", "host:path/to/repo", True),
        ("", "user@host.xz:~user/path/to/repo.git/", True),
        ("", "host.xz:~user/path/to/repo.git/", True),
        ("", "user@host.xz:path/to/repo.git", True),
        ("", "user@path/to/repo", True),
        ("", "host.xz:path/to/repo.git", True),
        ("", "rsync://host.xz/path/to/repo.git/", True),
        ("", "file:///path/to/repo.git/", True),
        ("", "file://~/path/to/repo.git/", True),
        ("", "github.com:containers/podman-compose.git", True),
        ("", "github:containers/podman-compose.git", True),
    ])
    def test_is_path_git_url(self, test_name: str, path: str, result: bool) -> None:
        self.assertEqual(is_context_git_url(path), result)



# SPDX-License-Identifier: GPL-2.0


import json
import os
import unittest

from tests.integration.test_utils import RunSubprocessMixin
from tests.integration.test_utils import podman_compose_path
from tests.integration.test_utils import test_path


class TestBuildLabels(unittest.TestCase, RunSubprocessMixin):
    def test_build_labels(self):
        """The build context can contain labels which should be added to the resulting image. They
        can be either an array or a map.
        """

        compose_path = os.path.join(test_path(), "build_labels/docker-compose.yml")

        try:
            self.run_subprocess_assert_returncode([
                podman_compose_path(),
                "-f",
                compose_path,
                "build",
                "test_build_labels_map",
                "test_build_labels_array",
            ])

            expected_labels = {
                "com.example.department": "Finance",
                "com.example.description": "Accounting webapp",
                "com.example.label-with-empty-value": "",
            }

            out, _ = self.run_subprocess_assert_returncode([
                "podman",
                "inspect",
                "my-busybox-build-labels-map",
                "my-busybox-build-labels-array",
            ])

            images = json.loads(out)
            self.assertEqual(len(images), 2)
            labels_map = images[0].get("Config", {}).get("Labels", {})
            labels_array = images[1].get("Config", {}).get("Labels", {})
            for k, v in expected_labels.items():
                self.assertIn(k, labels_map)
                self.assertEqual(labels_map[k], v)
                self.assertIn(k, labels_array)
                self.assertEqual(labels_array[k], v)

        finally:
            self.run_subprocess_assert_returncode([
                "podman",
                "rmi",
                "my-busybox-build-labels-map",
                "my-busybox-build-labels-array",
            ])



# SPDX-License-Identifier: GPL-2.0
# pylint: disable=protected-access
from __future__ import annotations

import argparse
import os
import unittest

import yaml
from parameterized import parameterized

from podman_compose import PodmanCompose
from podman_compose import normalize_final
from podman_compose import normalize_service_final

cwd = os.path.abspath(".")


class TestNormalizeFinalBuild(unittest.TestCase):
    cases_simple_normalization = [
        ({"image": "test-image"}, {"image": "test-image"}),
        (
            {"build": "."},
            {
                "build": {"context": cwd},
            },
        ),
        (
            {"build": "../relative"},
            {
                "build": {
                    "context": os.path.normpath(os.path.join(cwd, "../relative")),
                },
            },
        ),
        (
            {"build": "./relative"},
            {
                "build": {
                    "context": os.path.normpath(os.path.join(cwd, "./relative")),
                },
            },
        ),
        (
            {"build": "/workspace/absolute"},
            {
                "build": {
                    "context": "/workspace/absolute",
                },
            },
        ),
        (
            {
                "build": {
                    "dockerfile": "Dockerfile",
                },
            },
            {
                "build": {
                    "context": cwd,
                    "dockerfile": "Dockerfile",
                },
            },
        ),
        (
            {
                "build": {
                    "context": ".",
                },
            },
            {
                "build": {
                    "context": cwd,
                },
            },
        ),
        (
            {
                "build": {"context": "../", "dockerfile": "test-dockerfile"},
            },
            {
                "build": {
                    "context": os.path.normpath(os.path.join(cwd, "../")),
                    "dockerfile": "test-dockerfile",
                },
            },
        ),
        (
            {
                "build": {"context": ".", "dockerfile": "./dev/test-dockerfile"},
            },
            {
                "build": {
                    "context": cwd,
                    "dockerfile": "./dev/test-dockerfile",
                },
            },
        ),
    ]

    @parameterized.expand(cases_simple_normalization)
    def test_normalize_service_final_returns_absolute_path_in_context(self, input, expected):
        # Tests that [service.build] is normalized after merges
        project_dir = cwd
        self.assertEqual(normalize_service_final(input, project_dir), expected)

    @parameterized.expand(cases_simple_normalization)
    def test_normalize_returns_absolute_path_in_context(self, input, expected):
        project_dir = cwd
        compose_test = {"services": {"test-service": input}}
        compose_expected = {"services": {"test-service": expected}}
        self.assertEqual(normalize_final(compose_test, project_dir), compose_expected)

    @parameterized.expand(cases_simple_normalization)
    def test_parse_compose_file_when_single_compose(self, input, expected):
        compose_test = {"services": {"test-service": input}}
        dump_yaml(compose_test, "test-compose.yaml")

        podman_compose = PodmanCompose()
        set_args(podman_compose, ["test-compose.yaml"], no_normalize=None)

        podman_compose._parse_compose_file()

        actual_compose = {}
        if podman_compose.services:
            actual_compose = podman_compose.original_configuration(
                podman_compose.services["test-service"]
            )
        self.assertEqual(actual_compose, expected)

    @parameterized.expand([
        (
            {},
            {"build": "."},
            {"build": {"context": cwd}},
        ),
        (
            {"build": "."},
            {},
            {"build": {"context": cwd}},
        ),
        (
            {"build": "/workspace/absolute"},
            {"build": "./relative"},
            {
                "build": {
                    "context": os.path.normpath(os.path.join(cwd, "./relative")),
                }
            },
        ),
        (
            {"build": "./relative"},
            {"build": "/workspace/absolute"},
            {"build": {"context": "/workspace/absolute"}},
        ),
        (
            {"build": "./relative"},
            {"build": "/workspace/absolute"},
            {"build": {"context": "/workspace/absolute"}},
        ),
        (
            {"build": {"dockerfile": "test-dockerfile"}},
            {},
            {"build": {"context": cwd, "dockerfile": "test-dockerfile"}},
        ),
        (
            {},
            {"build": {"dockerfile": "test-dockerfile"}},
            {"build": {"context": cwd, "dockerfile": "test-dockerfile"}},
        ),
        (
            {},
            {"build": {"dockerfile": "test-dockerfile"}},
            {"build": {"context": cwd, "dockerfile": "test-dockerfile"}},
        ),
        (
            {"build": {"dockerfile": "test-dockerfile-1"}},
            {"build": {"dockerfile": "test-dockerfile-2"}},
            {"build": {"context": cwd, "dockerfile": "test-dockerfile-2"}},
        ),
        (
            {"build": "/workspace/absolute"},
            {"build": {"dockerfile": "test-dockerfile"}},
            {"build": {"context": "/workspace/absolute", "dockerfile": "test-dockerfile"}},
        ),
        (
            {"build": {"dockerfile": "test-dockerfile"}},
            {"build": "/workspace/absolute"},
            {"build": {"context": "/workspace/absolute", "dockerfile": "test-dockerfile"}},
        ),
        (
            {"build": {"dockerfile": "./test-dockerfile-1"}},
            {"build": {"dockerfile": "./test-dockerfile-2", "args": ["ENV1=1"]}},
            {
                "build": {
                    "context": cwd,
                    "dockerfile": "./test-dockerfile-2",
                    "args": ["ENV1=1"],
                }
            },
        ),
        (
            {"build": {"dockerfile": "./test-dockerfile-1", "args": ["ENV1=1"]}},
            {"build": {"dockerfile": "./test-dockerfile-2"}},
            {
                "build": {
                    "context": cwd,
                    "dockerfile": "./test-dockerfile-2",
                    "args": ["ENV1=1"],
                }
            },
        ),
        (
            {"build": {"dockerfile": "./test-dockerfile-1", "args": ["ENV1=1"]}},
            {"build": {"dockerfile": "./test-dockerfile-2", "args": ["ENV2=2"]}},
            {
                "build": {
                    "context": cwd,
                    "dockerfile": "./test-dockerfile-2",
                    "args": ["ENV1=1", "ENV2=2"],
                }
            },
        ),
    ])
    def test_parse_when_multiple_composes(self, input, override, expected):
        compose_test_1 = {"services": {"test-service": input}}
        compose_test_2 = {"services": {"test-service": override}}
        dump_yaml(compose_test_1, "test-compose-1.yaml")
        dump_yaml(compose_test_2, "test-compose-2.yaml")

        podman_compose = PodmanCompose()
        set_args(
            podman_compose,
            ["test-compose-1.yaml", "test-compose-2.yaml"],
            no_normalize=None,
        )

        podman_compose._parse_compose_file()

        actual_compose = {}
        if podman_compose.services:
            actual_compose = podman_compose.original_configuration(
                podman_compose.services["test-service"]
            )
        self.assertEqual(actual_compose, expected)


def set_args(podman_compose: PodmanCompose, file_names: list[str], no_normalize: bool) -> None:
    podman_compose.global_args = argparse.Namespace()
    podman_compose.global_args.file = file_names
    podman_compose.global_args.project_name = None
    podman_compose.global_args.env_file = None
    podman_compose.global_args.profile = []
    podman_compose.global_args.in_pod = "1"
    podman_compose.global_args.pod_args = None
    podman_compose.global_args.no_normalize = no_normalize


def dump_yaml(compose: dict, name: str) -> None:
    # Path(Path.cwd()/"subdirectory").mkdir(parents=True, exist_ok=True)
    with open(name, "w", encoding="utf-8") as outfile:
        yaml.safe_dump(compose, outfile, default_flow_style=False)


def test_clean_test_yamls() -> None:
    test_files = ["test-compose-1.yaml", "test-compose-2.yaml", "test-compose.yaml"]
    for file in test_files:
        if os.path.exists(file):
            os.remove(file)
